<?php
/**
 * classes/ibit_Action_EvaluationActivation.class.php
 *
 * @author  Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @author  Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author  Ivn Marcos <ivan.marcos@gmx.net>
 * @author  Nico Mueller <nico.mueller@uni-oldenburg.de>
 * @package IBIT_Evaluation
 * @copyright IBIT 2006-2008
 * @version 2.0.16
 */

/**
 * Die Evaluationsaktivierung erzeugt Evaluationen aus Vorlagen.
 *
 * Diese Klasse bernimmt das Aktivieren von Evaluationen bzw. das Umsetzen von Vorlagen in konkrete Evaluationen.
 *
 * @author     Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author     Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @package    IBIT_Evaluation
 * @subpackage Action
 */
class ibit_Action_EvaluationActivation
{
	/**
	 * Referenz auf das aufrufende Objekt
	 * @var objekt $parent
	 */
	var $parent;

	/**
	 * Konstruktor der Klasse.
	 * @param object $parent Referenz auf das aufrufende Objekt
	 */
	function ibit_Action_EvaluationActivation($parent)
	{
		$this->parent = $parent;
	}

	/**
	 * Zeigt das Men zur Aktivierung an.
	 * @uses ibit_EvaluationTemplate::getTemplatesInfo()
	 * @uses ibit_Helper::getInstituteObjects()
	 * @uses ibit_Helper::getSemester()
	 */
	function show()
	{
		$template_infos = ibit_EvaluationTemplate::getTemplatesInfo(true);
		$tmp_institute = ibit_Helper::getInstituteObjects();

		$institute = array();
		foreach ($tmp_institute as $tmp_institut)
		{
			$institut = array(
				'id'=>$tmp_institut->id,
				'name'=>$tmp_institut->name,
				'children'=>array()
			);

			$children = $tmp_institut->childinstitutes;
			foreach ($children as $child)
			{
				array_push($institut['children'], array(
					'id'=>$child->id,
					'name'=>$child->name
				));
			}
			array_push($institute, $institut);
		}

		$template = $this->parent->parent->getTemplate('admin_evaluation_activation');
		$template->set_attribute('template_infos', $template_infos);
		$template->set_attribute('semester_data', ibit_Helper::getSemester());
		$template->set_attribute('institute_data', $institute);
		$template->set_attribute('link_accept', $this->parent->base_link.'&command=activate');
		$template->set_attribute('link_cancel', $this->parent->parent_link);
		$template->set_attribute('semtypes', $GLOBALS['SEM_TYPE']);
		$template->set_attribute('link_preview', $this->parent->parent_link.'&section=templates&function=preview');
		echo $template->render();
	}

	/**
	 * Aktiviert eine Vorlage.
	 * @uses ibit_Evaluation
	 * @uses ibit_Helper::checkNumber()
	 * @uses ibit_Helper::getSeminarsForInstitute()
	 * @uses ibit_Helper::getFacultyForInstitute()
	 * @uses ibit_Helper::protectTemplate()
	 * @uses ibit_Helper::getInstituteObject()
	 */
	function activate()
	{
		$error = false;
		$error2 = Array();
		# Tag berprfen
		ibit_Helper::checkNumber($_POST['start_day'], 1, 31, 'Start-Tag', $error2);
		ibit_Helper::checkNumber($_POST['end_day'], 1, 31, 'Ende-Tag', $error2);
		# Monat berprfen
		ibit_Helper::checkNumber($_POST['start_month'], 1, 12, 'Start-Monat', $error2);
		ibit_Helper::checkNumber($_POST['end_month'], 1, 12, 'Ende-Monat', $error2);
		# Jahr berprfen
		ibit_Helper::checkNumber($_POST['start_year'], 2000, null, 'Start-Jahr', $error2);
		ibit_Helper::checkNumber($_POST['end_year'], 2000, null, 'Ende-Jahr', $error2);
		# Template-Id berprfen
		if (empty($_POST['activation_template_id']))
		{
			StudIPTemplateEngine::showErrorMessage('Bitte Evaluationsvorlage whlen');
			$error = true;
		}
		# Semester-Id berprfen
		if (empty($_POST['semester_id']))
		{
			StudIPTemplateEngine::showErrorMessage('Bitte gltiges Semester whlen');
			$error = true;
		}
		# Institut-Ids berprfen
		if (empty($_POST['institute']))
		{
			StudIPTemplateEngine::showErrorMessage('Bitte mindestens ein Institut auswhlen');
			$error = true;
		}
		#Veranstaltungsart berprfen
		if (empty($_POST['coursetype']))
		{
			StudIPTemplateEngine::showErrorMessage('Bitte mindestens ein Veranstaltungsart auswhlen');
			$error = true;
		}

		if ($error == true)
		{
			$this->show();
			return;
		}

		$start_time = mktime(0, 0, 0, $_POST['start_month'], $_POST['start_day'], $_POST['start_year']);
		$end_time = mktime(23, 59, 59, $_POST['end_month'], $_POST['end_day'], $_POST['end_year']);
		$eval_template_id = $_POST['activation_template_id'];
		$semester_id = $_POST['semester_id'];
		$coursetypes = $_POST['coursetype'];
		$filter = $_POST['filter'];

		//Veranstaltungstypen
		$semtypes = array();
		foreach ($GLOBALS['SEM_TYPE'] as $index=>$semtyp)
		{
			if($semtyp['class'] == 1)
			{
				array_push($semtypes, $index);
			}
		}

		//Institute
		$institute = array();
		foreach ($_POST['institute'] as $institute_id)
		{
			$inst = ibit_Helper::getInstituteObject($institute_id);
			array_push($institute, $inst->getId());

			$children = $inst->getAllChildInstitutes();
			foreach ($children as $child)
			{
				array_push($institute, $child->getId());
			}
		}

		// Kurse fr ausgewhlte Fakultten und Institute laden
		$institute = array_unique($institute); //doppelte eintrge entfernen
		$courses = array();
		$visible_entries = 0;

		foreach ($institute as $institute_id)
		{
			$tmp_courses = ibit_Helper::getSeminarsForInstitute($institute_id, $semester_id);

			#echo "VORHER<pre>";
			#print_r($tmp_courses);
			#echo "</pre>";

			$faculty_id = ibit_Helper::getFacultyForInstitute($institute_id);

			foreach ($tmp_courses as $course)
			{
				//versteckter Kurs
				if($course['visible'] == 0)
				{
					$visible_entries += 1;
				}
				//nicht versteckter Kurs bercksichtigen!
				if($course['visible'] == 1)
				{
					//Typ berprfen, d.h. Veranstaltung aus Lehre
					if (in_array($course['typ'], $coursetypes) )
					{
						//Wenn Grundstudium oder Hauptstudium ausgewhlt ist
						if($course['typ'] == 2 || $course['typ'] == 3)
						{
							//Filter berprfung
							//Nur Seminare
							if (in_array('se', $filter))
							{
								if(stripos($course['titel'], "Seminar") !== false)
								{
									$course['institute_id'] = $institute_id;
									$course['faculty_id'] = $faculty_id;
									array_push($courses, $course);
								}
							}
							//nur bungen
							elseif (in_array('ue', $filter))
							{
								if(stripos($course['titel'], "bung") !== false)
								{
									$course['institute_id'] = $institute_id;
									$course['faculty_id'] = $faculty_id;
									array_push($courses, $course);
								}
							}
							//wo nicht bung und seminar vorkommt
							elseif (in_array('no', $filter))
							{
								if(stripos($course['titel'], "bung") === false && stripos($course['titel'], "Seminar") === false)
								{
									$course['institute_id'] = $institute_id;
									$course['faculty_id'] = $faculty_id;
									array_push($courses, $course);
								}
							}
							//kein filter gewhlt
							else
							{
								$course['institute_id'] = $institute_id;
								$course['faculty_id'] = $faculty_id;
								array_push($courses, $course);
							}
						}
						//ansonsten einfgen
						else
						{
							$course['institute_id'] = $institute_id;
							$course['faculty_id'] = $faculty_id;
							array_push($courses, $course);
						}
					}
					//Alle anderen - ohne berprfung des Titels...
					elseif(in_array( 'other', $coursetypes) && !in_array($course['typ'], $semtypes))
					{
						$course['institute_id'] = $institute_id;
						$course['faculty_id'] = $faculty_id;
						array_push($courses, $course);
					}
				}
			}
		}

		#echo "NACHHER<pre>";
		#print_r($courses);
		#echo "</pre>";

		// Evals erstellen und in die DB speichern
		$course_done = array();
		$entries = 0;
		foreach ($courses as $course)
		{
			if (in_array($course['id'], $course_done))
			{
				continue;
			}
			array_push($course_done, $course['id']);

			$evaluation = new ibit_Evaluation();
			$evaluation->setTemplateId($eval_template_id);
			$evaluation->setSeminarId($course['id']);
			$evaluation->setInstitutId($course['institute_id']);
			$evaluation->setFakultaetId($course['faculty_id']);
			$evaluation->setStatus(1);
			$evaluation->setConfirmed(0);
			$evaluation->setSemesterId($semester_id);
			$evaluation->setStartDate($start_time);
			$evaluation->setStopDate($end_time);
			$evaluation->setCreateAuthorId($_SESSION['auth']->auth['uid']);
			$evaluation->setChangeAuthorId($_SESSION['auth']->auth['uid']);
			$evaluation->save();

			// Insert tutors into user table
			$tutor_ids = ibit_EvaluationSeminar::getTutorsForSeminar($course['id']);
			foreach ($tutor_ids as $tutor_id)
			{
				DBManager::Get()->exec("INSERT INTO ibit_evaluation_user (evaluation_id, user_id, timestamp, hidden) VALUES ('{$evaluation->getId()}', '{$tutor_id}', NOW(), 1)");
			}

			$entries += 1;
		}

		//Das Eval-Template gegen Vernderungen sperren, da in Benutzung.
		ibit_Helper::protectTemplate($eval_template_id);

		$this->parent->parent->reportSuccess('Die Evaluation wurde erfolgreich in <b>'.$entries.'</b> Veranstaltungen eingetragen. <b>'.$visible_entries.'</b> versteckte Veranstaltungen wurden ignoriert.');
		$this->show();
	}
}
?>