<?php
/**
 * classes/ibit_Action_Handler_Admin.class.php
 *
 * @author  Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @author  Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author  Ivn Marcos <ivan.marcos@gmx.net>
 * @author  Nico Mueller <nico.mueller@uni-oldenburg.de>
 * @package IBIT_Evaluation
 * @copyright IBIT 2006-2008
 * @version 2.0.15
 */

/**
 * Diese Klasse enthlt alle Funktionen, die ein Admin ausfhren kann.
 *
 * @author     Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author     Ivn Marcos <ivan.marcos@gmx.net>
 * @author     Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @package    IBIT_Evaluation
 * @subpackage Action_Handler
 * @copyright  Copyright 2006, 2007, 2008 IBIT
 * @version    2.0.9
 */
class ibit_Action_Handler_Admin extends ibit_Action_Handler
{
	/**
	 * Id des Admins
	 * @var string $user_id
	 */
	var $user_id;

	/**
	 * Konstruktor der Klasse.
	 * Initialisiert die Klasse, indem der Konstruktor der Elternklasse mit dem bergebenen aufrufenden Objekt
	 * und den Standardwerten fr Sektion und Funktion aufgerufen wird.
	 * @param object $parent Referenz auf das aufrufende Objekt
	 */
	function ibit_Action_Handler_Admin( $parent, $user_id )
	{
		parent::ibit_Action_Handler( $parent, 'evaluation', 'administration' );
		$this->user_id = $user_id;
	}

	/**
	 * Diese Funktion zeigt das Hauptmen der Administrationssicht an.
	 * @uses ibit_EvaluationTemplate::getTemplatesInfo()
	 */
	function templates_index()
	{
		$template = $this->parent->getTemplate('admin_menu.php');
		$template->set_attribute('link_newtemplate', PluginEngine::getLink($this->parent, array('section'=>'templates', 'function'=>'create', 'new'=>'true')));
		$template->set_attribute('link_edittemplate', PluginEngine::getLink($this->parent, array('section'=>'templates', 'function'=>'edit')));
		$template->set_attribute('link_previewtemplate', PluginEngine::getLink($this->parent, array('section'=>'templates', 'function'=>'preview')));
		$template->set_attribute('link_copytemplate', PluginEngine::getLink($this->parent, array('section'=>'templates', 'function'=>'copy')));
		$template->set_attribute('link_delete_section', PluginEngine::getLink($this->parent, array('section'=>'templates')));
		$template->set_attribute('alltemplates', ibit_EvaluationTemplate::getTemplatesInfo());
		echo $template->render();
	}

	/**
	 * Leitet das Erstellen einer neuen Evaluationsvorlage ein.
	 */
	function templates_create()
	{
		if (!empty($_REQUEST['new']) and $_REQUEST['new']=='true')
		{
			unset($_SESSION['template']);
		}
		$this->_edit_template(null);
	}

	/**
	 * Leitet das Bearbeiten einer Evaluationsvorlage ein.
	 */
	function templates_edit()
	{
		$template_id = empty($_REQUEST['template_id'])?null:$_REQUEST['template_id'];
		$this->_edit_template($template_id);
	}

	/**
	 * Generelle Funktion zum Bearbeiten von Vorlagen.
	 * @param int $template_id Template-Id, die bearbeitet werden soll oder NULL zum Erzeugen einer neuen Evaluation
	 * @uses ibit_Action_EvaluationEditTemplate
	 * @uses ibit_EvaluationAnswer::getAnswerInfos()
	 */
	function _edit_template($template_id)
	{
		$edit = new ibit_Action_EvaluationEditTemplate($this, $template_id);

		$command = empty($_REQUEST['command'])?'':trim($_REQUEST['command']);

		switch ($command)
		{
			# Speichern
			case 'save':
				$edit->save();
			return;
			# Vorlageninformationen aktualisieren
			case 'template_metadata':
				$edit->metadata();
			break;
			# Neue Gruppe anlegen
			case 'create_group':
				$edit->create_group();
			break;
			# Gruppe bearbeiten
			case 'edit_group':
				$edit->edit_group();
			break;
			# Gruppe bearbeiten - Verarbeitung
			case 'edit_group_confirm':
				$edit->edit_group_confirm();
			break;
			# Gruppe verschieben
			case 'change_group_position':
				$edit->change_group_position();
			break;
			# Gruppe lschen
			case 'delete_group':
				$edit->delete_group();
			break;
			# Neue Frage anlegen
			case 'create_question':
				$GLOBALS['new_question_position'] = $_REQUEST['group'];
			break;
			# Frage bearbeiten
			case 'edit_question':
				$edit->edit_question();
			break;
			# Frage bearbeiten - Verarbeitung
			case 'edit_question_confirm':
				$edit->edit_question_confirm();
			break;
			# Frage verschieben
			case 'change_question_position':
				$edit->change_question_position();
			break;
			# Frage lschen
			case 'delete_question':
				$edit->delete_question();
			break;
			# Neue Frage hinzufgen
			case 'create_question_confirm':
				$edit->create_question();
			break;
			# Default
			default:
			break;
		}

		$no_display = array('edit_question', 'edit_group', 'save');
		if (!in_array($command, $no_display))
		{
			$edit->display();
		}
	}

	/**
	 * Zeigt die Vorschau einer Evaluationsvorlage an.
	 * @uses ibit_EvaluationTemplate
	 */
	function templates_preview()
	{
		$template_id = $_REQUEST['template_id'];

		if (empty($template_id) or !is_numeric($template_id))
		{
			$this->parent->reportError('Vorlage kann nicht angezeigt werden');
			return;
		}
		$eval_template = new ibit_EvaluationTemplate($template_id);

		$template = $this->parent->getTemplate('common_display_template.php');
		$template->set_attribute('template', $eval_template->flattenToarray());
		$template->set_attribute('link_back', PluginEngine::getLink($this->parent));
		echo $template->render();
	}

	/**
	 * Kopiert eine Evaluationsvorlage.
	 * @uses ibit_EvaluationTemplate
	 */
	function templates_copy()
	{
		$template_id = empty($_REQUEST['template_id'])?'':$_REQUEST['template_id'];

		if( !empty($template_id) and is_numeric($template_id))
		{
			# zu kopierende Vorlage laden
			$template = new ibit_EvaluationTemplate($template_id);
			$template->loadStructure();

			# Kopie erstellen
			$template->setTitle($template->getTitle().' Kopie');
			$template->setTemplateID(null);
			$template->setProtected(0);
			$template->setStatus(0);
			$template->setAuthorId($_SESSION['auth']->auth['uid']);
			# Kopie speichern
			$template->storeStructure();

			$this->parent->reportSuccess('Die Kopie wurde erfolgreich erstellt und gespeichert');
		}
		else
		{
			$this->parent->reportError('Es konnte keine Kopie erstellt werden');
		}
		$this->templates_index();
	}

	/**
	 * Lscht eine Evaluationsvorlage.
	 * @uses ibit_EvaluationTemplate::delete()
	 */
	function templates_delete()
	{
		$template_id = $_REQUEST['template_id'];

		if (ibit_EvaluationTemplate::delete($template_id))
		{
			$this->parent->reportSuccess('Vorlage wurde erfolgreich gelscht');
		}
		else
		{
			$this->parent->reportError('Vorlage konnte nicht gelscht werden');
		}
		$this->templates_index();
	}

	/**
	 * Zeigt und verwaltet die Evaluationsverwaltung.
	 * @uses ibit_Action_EvaluationAdministration
	 * @uses ibit_Evaluation::delete()
	 */
	function evaluation_administration()
	{
		$command = empty($_REQUEST['command'])?'':$_REQUEST['command'];
		$action = empty($_REQUEST['action'])?'':$_REQUEST['action'];

		$administration = new ibit_Action_EvaluationAdministration($this);

		switch ($action)
		{
			# Evaluation lschen
			case 'delete_evaluations':
				if (ibit_Evaluation::delete($_REQUEST['evaluation_ids']))
				{
					$this->parent->reportSuccess('Die Evaluationen wurden erfolgreich gelscht');
				}
				else
				{
					$this->parent->reportError('Beim Lschen der Evaluationen ist ein Fehler aufgetreten');
				}
			break;
			# Evaluationen eines Instituts lschen
			case 'delete_evaluations_by_institute_ids':
				$institute_ids = $_REQUEST['institute_ids'];
				if ($count=$administration->deleteEvaluationsByInstituteId($institute_ids))
				{
					$this->parent->reportSuccess('Die '.$count.' Evaluationen der Institute wurden erfolgreich gelscht');
				}
				else
				{
					$this->parent->reportError('Beim Lschen der Evaluationen ist ein Fehler aufgetreten');
				}
			break;
		}

		$no_header_display = array('edit_evaluation', 'evaluation_details');
		if (!in_array($command, $no_header_display))
		{
			$administration->displayHeader($command);
		}

		switch($command)
		{
			# Evaluation anzeigen
			case 'evaluation_details':
				$administration->evaluation_details($_REQUEST['evaluation_id']);
			break;
			# Evaluation bearbeiten
			case 'edit_evaluation':
				$administration->edit_evaluation($_REQUEST['evaluation_id']);
			break;
			# Laufende Evaluationen einer Fakultt anzeigen
			case 'show_faculty':
				$administration->show_faculty($_REQUEST['faculty_id']);
			break;
			# Laufende Evaluationen eines Institutes anzeigen
			case 'show_institute':
				$administration->show_institute($_REQUEST['institute_id']);
			break;
			# bersicht aller Fakultten zeigen
			case 'show_faculties':
			default:
				$administration->show_overview();
			break;
		}
	}

	/**
	 * Zeigt und verwaltet die Evaluationsaktivierung.
	 * @uses ibit_Action_EvaluationActivation
	 */
	function evaluation_activation()
	{
		$command = empty($_REQUEST['command'])?'':$_REQUEST['command'];
		$activation = new ibit_Action_EvaluationActivation($this, $command);

		switch($command)
		{
			# Verarbeitung
			case 'activate':
				$activation->activate();
			break;
			# Anzeige
			case 'show':
			default:
				$activation->show();
			break;
		}
	}

	function evaluation_unlock()
	{
		$command = empty($_REQUEST['command'])?'':$_REQUEST['command'];
		$visible = new ibit_Action_EvaluationMakeVisible($this);

		switch($command)
		{
			# Verarbeitung
			case 'unlock':
				$visible->makeVisible();
			break;
			# Anzeige
			default:
				$visible->show();
			break;
		}
	}

	/**
	 * Zeigt und verwaltet die Evaluationsaktivierung fr ein einzelnes Seminar.
	 * @uses ibit_Action_EvaluationActivation
	 */
	function evaluation_activation_seminar()
	{
		$command = empty($_REQUEST['command'])?'':$_REQUEST['command'];
		$activation = new ibit_Action_EvaluationActivationSeminar($this, $command);

		switch($command)
		{
			# Verarbeitung
			case 'activate':
				$activation->activate();
			break;
			# Suche erfolgreich
			case 'searched':
				$activation->searched();
			break;
			# Anzeige
			case 'show':
			default:
				$activation->show();
			break;
		}
	}

	/**
	 * Zeigt und verwaltet den XML-Export.
	 * @uses ibit_Action_EvaluationXmlExport
	 */
	function tools_xml_export()
	{
		$command = empty($_REQUEST['command'])?'':$_REQUEST['command'];
		$xml_export = new ibit_Action_EvaluationXmlExport($this);

		switch ($command)
		{
			# Exportieren
			case 'export':
				$xml_export->export();
			break;
			# Men anzeigen
			default:
				$xml_export->showMenu();
			break;
		}
	}

	/**
	 * Zeigt und verwaltet allgmeine Einstellungen zum Plugin
	 */
	 function tools_configure()
	{
		$command = empty($_REQUEST['command'])?'':$_REQUEST['command'];
		$config = new ibit_Action_EvaluationConfiguration($this);

		switch ($command)
		{
			# Einstellungen anzeigen
			default:
				$config->showSettings();
			break;
		}
	}

	/**
	 * Gibt eine Liste zuflliger Teilnehmer aus
	 */
	 function tools_randomize()
	{
		$command = empty($_REQUEST['command'])?'':$_REQUEST['command'];
		$randomize = new ibit_Action_EvaluationRandomize($this);

		switch ($command)
		{
			case 'result':
				$randomize->showResults();
			break;
			default:
				$randomize->showSettings();
			break;
		}
	}
}
?>