<?php
/**
 * classes/ibit_Evaluation.class.php
 *
 * @author  Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @author  Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author  Ivn Marcos <ivan.marcos@gmx.net>
 * @author  Nico Mueller <nico.mueller@uni-oldenburg.de>
 * @package IBIT_Evaluation
 * @copyright Copyright 2006, 2007, 2008 IBIT
 * @version 2.0.9
 */

/**
 * Die Reprsentation einer Evaluation.
 *
 * Diese Klasse stellt eine Reprsentation der Daten einer Evaluation dar. Die Daten einer Evaluation knnen hier geladen, gespeichert oder
 * verndert werden. Ebenso werden statische Funktionen zum Auslesen verschiedener Evaluationen angeboten.
 *
 * @author     Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author     Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @package    IBIT_Evaluation
 * @subpackage Structure
 * @copyright  Copyright 2006, 2007, 2008 IBIT
 * @version    2.0.9
 */
class ibit_Evaluation
{
	/**
	 * Die ID der Evaluation
	 * @var string $evaluation_id
	 */
	var $evaluation_id = null;

	/**
	 * Die Template-ID der Evaluation
	 * @var string $template_id
	 */
	var $template_id;

	/**
	 * Das Template-Objekt der Evaluation
	 * @var object $template
	 */
	var $template;

	/**
	 * ID des zur Evaluation gehrenden Seminars
	 * @var string $seminar_id
	 */
	var $seminar_id;

	/**
	 * ID des zur Evaluation gehrenden Instituts
	 * @var string $institute_id
	 */
	var $institute_id;

	/**
	 * ID der zur Evaluation gehrenden Fakultt
	 * @var string $faculty_id
	 */
	var $faculty_id;

	/**
	 * ID des zur Evaluation gehrenden Semesters
	 * @var string $semester_id
	 */
	var $semester_id;

	/**
	 * Objekt des zur Evaluation gehrenden Seminars
	 * @var mixed $seminar
	 */
	var $seminar;

	/**
	 * Der Status der Evaluation
	 * @var int $status
	 */
	var $status;

	/**
	 * Gibt an, ob die Evaluation freigegeben wurde
	 * @var boolean $confirmed
	 */
	var $confirmed;

	/**
	 * Das Startdatum der Evaluation
	 * @var int $start_date
	 */
	var $start_date;

	/**
	 * Das Enddatum der Evaluation
	 * @var int $stop_date
	 */
	var $stop_date;

	/**
	 * Das Datum der letzten nderung
	 * @var int $change_date
	 */
	var $change_date;

	/**
	 * Die Id des Autoren der letzten nderung
	 * @var string $change_author_id
	 */
	var $change_author_id;

	/**
	 * Das Erstellungsdatum der Evaluation
	 * @var int $create_date
	 */
	var $create_date;

	/**
	 * Die Id des Autoren, der die Evaluation erstellt hat
	 * @var string $create_author_id
	 */
	var $create_author_id;

	/**
	 * Das Datum, an dem die Evaluation freigeschaltet worden ist oder null
	 * @var int $visible_date
	 */
	var $visible_date;

	/**
	 * Die Id des Users, der die Evaluation freigeschaltet hat oder null
	 * @var string $visible_author
	 */
	var $visible_author_id;

	/**
	 * Der Kommentar der Evaluation
	 * @var string $comment
	 */
	var $comment;

	/**
	 * Die Anzahl der Teilnehmer
	 * @var string $participants
	 */
	var $participants;

	/**
	 * Konstruktor der Klasse.
	 * @param int $evaluation_id Id der Evaluation
	 */
	function ibit_Evaluation( $evaluation_id = null )
	{
		if ( $evaluation_id !== null )
		{
			$this->load( $evaluation_id );
		}
	}

	/**
	 * Ldt die Daten der Evaluation mit der bergebenen Id in dieses Objekt.
	 * @param string $eval_id Id der Evaluation
	 * @return boolean Statusmeldung
	 */
	function load( $eval_id )
	{
		$this->evaluation_id = $eval_id;

		$row = DBManager::Get()->query("SELECT e.evaluation_template_id, seminar_id, institut_id, fakultaet_id, semester_id, UNIX_TIMESTAMP(start_date) AS start_date, UNIX_TIMESTAMP(stop_date) AS stop_date, UNIX_TIMESTAMP(create_date) AS create_date, create_author_id, UNIX_TIMESTAMP(change_date) AS change_date, change_author_id, status, confirmed, visible_date, visible_author_id, comment, SUM(u.hidden<>1) AS participants FROM ibit_evaluation AS e LEFT JOIN ibit_evaluation_user AS u ON (e.evaluation_id=u.evaluation_id) WHERE e.evaluation_id = '{$this->evaluation_id}' GROUP BY e.evaluation_id")->fetch(PDO::FETCH_ASSOC);

		if (empty($row))
		{
			return false;
		}
		$this->seminar_id = $row['seminar_id'];
		$this->institute_id = $row['institut_id'];
		$this->faculty_id = $row['fakultaet_id'];
		$this->template_id = $row['evaluation_template_id'];
		$this->semester_id = $row['semester_id'];
		$this->comment = $row['comment'];
		$this->status = $row['status'];
		$this->confirmed = $row['confirmed'];
		$this->start_date = $row['start_date'];
		$this->stop_date = $row['stop_date'];
		$this->create_date = $row['create_date'];
		$this->create_author_id = $row['create_author_id'];
		$this->change_date = $row['change_date'];
		$this->change_author_id = $row['change_author_id'];
		$this->visible_date = $row['visible_date'];
		$this->visible_author_id = $row['visible_author_id'];
		$this->participants = $row['participants'];

		return true;
	}

	/**
     * Speichert die Daten der Evaluation in der Datenbank.
     */
	function save()
	{
		//sonst gehen andere abfragen nicht mehr...
		if(empty($this->visible_date))
		{
			$visible_date = 'NULL';
		}
		else 
		{
			$visible_date = "FROM_UNIXTIME({$this->visible_date})";
		}
		
		if(empty($this->comment))
		{
			$comment = 'NULL';
		}
		else
		{
			$comment = "'{$this->comment}'";
		}

		$sql = "INSERT INTO ibit_evaluation (evaluation_id, evaluation_template_id, seminar_id, institut_id, fakultaet_id, semester_id, start_date, stop_date, change_date, change_author_id, `status`, confirmed, visible_date, visible_author_id, comment, create_date, create_author_id) VALUES ('{$this->evaluation_id}', '{$this->template_id}', '{$this->seminar_id}', '{$this->institute_id}', '{$this->faculty_id}', '{$this->semester_id}', FROM_UNIXTIME({$this->start_date}), FROM_UNIXTIME({$this->stop_date}), NOW(), '{$this->change_author_id}', '{$this->status}', '{$this->confirmed}', {$visible_date}, '{$this->visible_author_id}', $comment, NOW(), '{$this->create_author_id}') ON DUPLICATE KEY UPDATE evaluation_template_id=VALUES(evaluation_template_id), seminar_id=VALUES(seminar_id), institut_id=VALUES(institut_id), fakultaet_id=VALUES(fakultaet_id), semester_id=VALUES(semester_id), start_date=VALUES(start_date), stop_date=VALUES(stop_date), change_date=VALUES(change_date), change_author_id=VALUES(change_author_id), status=VALUES(status), confirmed=VALUES(confirmed), visible_date=VALUES(visible_date), visible_author_id=VALUES(visible_author_id), comment=VALUES(comment)";
		$result = DBManager::Get()->exec($sql);
		if ($result and $this->evaluation_id === null)
		{
			$this->evaluation_id = DBManager::Get()->lastInsertId();
		}
	}

	/**
	 * Lscht eine Evaluation oder eine Menge von Evaluationen und smtliche Querverweise aus der Datenbank.
	 * @param mixed $evaluation_ids Id(s) der Evaluation(en)
	 * @return boolean Statusmeldung
	 */
	function delete( $evaluation_ids )
	{
		if ( empty( $evaluation_ids ) )
		{
			return false;
		}

		$deleted_evaluations = DBManager::Get()->exec("DELETE QUICK FROM ibit_evaluation WHERE evaluation_id IN ('".implode("','", (array)$evaluation_ids)."')");
		DBManager::Get()->exec("DELETE QUICK FROM ibit_evaluation_answers WHERE evaluation_id IN ('".implode("','", (array)$evaluation_ids)."')");
		DBManager::Get()->exec("DELETE QUICK FROM ibit_evaluation_user WHERE evaluation_id IN ('".implode("','", (array)$evaluation_ids)."')");
		DBManager::Get()->exec("DELETE QUICK FROM ibit_evaluation_summary_views WHERE evaluation_id IN ('".implode("','", (array)$evaluation_ids)."')");

		#DBManager::Get()->exec("OPTIMIZE TABLE ibit_evaluation, ibit_evaluation_answers, ibit_evaluation_user, ibit_evaluation_summary_views");

		return $deleted_evaluations > 0;
	}

	/**
	 * Liefert die Id der Evaluation zurck.
	 * @return string Id der Evaluation
	 */
	function getEvaluationId()
	{
		return $this->evaluation_id;
	}

	/**
	 * Liefert die Id der Evaluation zurck.
	 * @return string Id der Evaluation
	 */
	function getId()
	{
		return $this->evaluation_id;
	}

	/**
	 * Liefert die Vorlagen-Id der Evaluation zurck.
	 * @return string Id der Vorlage
	 */
	function getTemplateId()
	{
		return $this->template_id;
	}

	/**
	 * Setzt die Vorlagen-Id der Evaluation.
	 * @param $template_id Id der Vorlage
	 */
	function setTemplateId( $template_id )
	{
		$this->template_id = $template_id;
	}

	/**
	 * Ldt die zur Evaluation gehrende Vorlage
	 * @return boolean Statusmeldung
	 * @uses ibit_EvaluationTemplate
	 */
	function loadTemplate()
	{
		if ( $this->template_id === null )
		{
			return false;
		}

		$this->template = new ibit_EvaluationTemplate( $this->template_id );
		return true;
	}

	/**
	 * Liefert die Id des zur Evaluation gehrenden Seminars zurck.
	 * @return string Id des Seminars
	 */
	function getSeminarId()
	{
		return $this->seminar_id;
	}

	/**
	 * Setzt die Id des zur Evaluation gehrenden Seminars.
	 * @param string $seminar_id Id des Seminars
	 */
	function setSeminarId( $seminar_id )
	{
		$this->seminar_id = $seminar_id;
	}

	/**
	 * Ldt das zur Evaluation gehrende Seminar.
	 * @return boolean Statusmeldung
	 * @uses ibit_EvaluationSeminar
	 */
	function loadSeminar()
	{
		if ( $this->seminar_id === null)
		{
			return false;
		}

		$this->seminar = new ibit_EvaluationSeminar( $this->seminar_id );
		return true;
	}

	/**
	 * Liefert die Id des zur Evaluation gehrenden Instituts zurck.
	 * @return string Id des Instituts
	 */
	function getInstitutId()
	{
		return $this->institute_id;
	}

	/**
	 * Setzt die Id des zur Evaluation gehrenden Instituts.
	 * @param string $institute_id Id des Instituts
	 */
	function setInstitutId( $institute_id )
	{
		$this->institute_id = $institute_id;
	}

	/**
	 * Liefert die Id der zur Evaluation gehrenden Fakultt zurck.
	 * @return string Id der Fakultt
	 */
	function getFakultaetId()
	{
		return $this->faculty_id;
	}

	/**
	 * Setzt die Id der zur Evaluation gehrenden Fakultt.
	 * @param string $faculty_id Id der Fakultt
	 */
	function setFakultaetId( $faculty_id )
	{
		$this->faculty_id = $faculty_id;
	}

	/**
	 * Liefert die Id des zur Evaluation gehrenden Semesters zurck.
	 * @return string Id des Semesters
	 */
	function getSemesterId()
	{
		return $this->semester_id;
	}

	/**
	 * Setzt die Id des zur Evaluation gehrenden Semesters.
	 * @param string $semester_id Id des Semesters
	 */
	function setSemesterId( $semester_id )
	{
		$this->semester_id = $semester_id;
	}

	/**
	 * Liefert den Status der Evaluation zurck.
	 * @return int Status der Evaluation
	 */
	function getStatus()
	{
		return $this->status;
	}

	/**
	 * Setzt den Status der Evaluation.
	 * @param int $status
	 */
	function setStatus( $status )
	{
		$this->status = $status;
	}

	/**
	 * Liefert den Besttigt-Status der Evaluation zurck.
	 * @return string Besttigt-Status der Evaluation
	 */
	function getConfirmed()
	{
		return $this->confirmed;
	}

	/**
	 * Setzt den Besttigt-Status der Evaluation.
	 * @param string $confirmed Neuer Besttigt-Status
	 */
	function setConfirmed( $confirmed )
	{
		$this->confirmed = $confirmed;
	}

	/**
	 * Setzt den Kommentar der Evaluation.
	 * @param string $comment
	 */
	function setComment( $comment )
	{
		$this->comment = $comment;
	}

	/**
	 * Liefert den Kommentar der Evaluation zurck.
	 * @return string Kommentar der Evaluation
	 */
	function getComment()
	{
		return $this->comment;
	}

	/**
	 * Liefert das Startdatum der Evaluation zurck.
	 * @return int Startdatum der Evaluation als Unix-Timestamp.
	 */
	function getStartDate()
	{
		return $this->start_date;
	}

	/**
	 * Setzt das Startdatum der Evaluation
	 * @param mixed $start_date Startdatum der Evaluation als Unix-Timestamp
	 */
	function setStartDate( $start_date = null )
	{
		if ( $start_date === null )
		{
			$start_date = time();
		}
		elseif ( !is_int( $start_date ) )
		{
			$start_date = strtotime( $start_date );
		}
		$this->start_date = $start_date;
	}

	/**
	 * Liefert das Enddatum der Evaluation zurck.
	 * @return int Enddatum der Evaluation als Unix-Timestamp
	 */
	function getStopDate()
	{
		return $this->stop_date;
	}

	/**
	 * Setzt das Enddatum der Evaluation.
	 * @param mixed $stop_date Enddatum der Evaluation als Unix-Timestamp.
	 */
	function setStopDate( $stop_date = null )
	{
		if ( $stop_date === null )
		{
			$stop_date = time();
		}
		elseif ( !is_int( $stop_date ) )
		{
			$stop_date = strtotime( $stop_date );
		}
		$this->stop_date = $stop_date;
	}

	/**
	 * Liefert das nderungsdatum der Evaluation zurck.
	 * @return int nderungsdatum der Evaluation als Unix-Timestamp
	 */
	function getChangeDate()
	{
		return $this->change_date;
	}

	/**
	 * Liefert die Autoren-Id der letzten nderung zurck.
	 * @return string Id des Autoren
	 */
	function getChangeAuthorId()
	{
		return $this->change_author_id;
	}

	/**
	 * Setzt die Autoren-Id der letzten nderung.
	 * @param String $author_id Id des Autoren.
	 */
	function setChangeAuthorId( $change_author_id )
	{
		$this->change_author_id = $change_author_id;
	}

	/**
	 * Liefert das Erstellungsdatum der Evaluation zurck.
	 * @return int Erstellungsdatum der Evaluation als Unix-Timestamp
	 */
	function getCreateDate()
	{
		return $this->create_date;
	}

	/**
	 * Liefert die Autoren-Id des Users zurck, der die Evaluation erstellt hat.
	 * @return string Id des Autors
	 */
	function getCreateAuthorId()
	{
		return $this->create_author_id;
	}

	/**
	 * Setzt die Autoren-Id der Evaluation.
	 * @param String $author_id Id des Autors
	 */
	function setCreateAuthorId( $create_author_id )
	{
		$this->create_author_id = $create_author_id;
	}

	/**
	 * Liefert zurck, ob die Evaluation freigeschaltet ist oder nicht.
	 * @return boolean Evaluation freigeschaltet?
	 */
	function isVisible()
	{
		return $this->visible_date !== null;
	}

	/**
	 * Liefert die Id des Users zurck, der die Evaluation freigeschaltet hat.
	 * @return string Id des Users
	 */
	function getVisibleAuthorId()
	{
		return $this->visible_author_id;
	}

	/**
	 * Liefert das Datum zurck, an dem die Evaluation freigeschaltet wurde.
	 * @return int Freischalatungsdatum der Evaluation
	 */
	function getVisibleDate()
	{
		return $this->visible_date;
	}

	/**
	 * Schaltet eine Evaluation frei oder nimmt diese Freischaltung zurck.
	 * @param boolean $state Freigeschaltet?
	 * @param string $author Id des Users, der die nderung vornimmt
	 */
	function setVisible( $state, $visible_author_id )
	{
		if ($state)
		{
			$this->visible_date = time();
			$this->visible_author_id = $visible_author_id;
		}
		else
		{
			$this->visible_date = null;
			$this->visible_author_id = null;
		}
	}

	/**
	 * Liefert die Anzahl der Teilnehmer zurck.
	 * @return int Anzahl der Teilnehmer
	 */
	function getNumberOfParticipants()
	{
		return $this->participants;
	}

	/**
	 * Bricht das Objekt in ein mehrdimensionales array auf, das alle relevanten Informationen enthlt.
	 * Das array hat dabei folgenden Aufbau:
	 *
	 * <pre>
	 *   [
	 *     evaluation_id    => Id der Evaluation,
	 *     template_id      => Id der Vorlage,
	 *     seminar_id       => Id des Seminars,
	 *     institut_id      => Id des Instituts,
	 *     fakultaet_id     => Id der Fakultaet,
	 *     status           => Status der Evaluation,
	 *     confirm          => Besttigt-Status der Evaluation (nicht benutzt),
	 *     comment          => Kommentar der Evaluation,
	 *     semester_id      => Id des Semesters,
	 *     start_date       => Startdatum der Evaluation,
	 *     stop_date        => Enddatum der Evaluation,
	 *     change_data      => nderungsdatum der Evaluation,
	 *     change_author_id => Id des Users, der die letzte nderung durchgefhrt hat,
	 *     create_data      => Erstelldatum der Evaluation,
	 *     create_author_id => Id des Users, der die Evaluation erstellt hat,
	 *     visible_data     => Freischaltdatum der Evaluation,
	 *     visible_author   => Id des Users, der die Evaluation freigeschaltet hat,
	 *     participants     => Anzahl der User, die an der Evaluation teilgenommen haben,
	 *     template         => Vorlagendaten [
	 *                           ...
	 *                                       ],
	 *     seminar          => Seminardaten [
	 *                           ...
	 *                                      ]
	 *    ]
	 * </pre>
	 *
	 * @return array Das Objekt als array
	 * @uses ibit_EvaluationTemplate::flattenToarray()
	 * @uses ibit_EvaluationSeminar::flattenToarray();
	 */
	function flattenToarray()
	{
		$result = array();

		$result['evaluation_id'] = $this->evaluation_id;
		$result['template_id'] = $this->template_id;
		$result['seminar_id'] = $this->seminar_id;
		$result['institut_id'] = $this->institute_id;
		$result['fakultaet_id'] = $this->faculty_id;
		$result['status'] = $this->status;
		$result['confirmed'] = $this->confirmed;
		$result['comment'] = $this->comment;
		$result['semester_id'] = $this->semester_id;
		$result['start_date'] = $this->start_date;
		$result['stop_date'] = $this->stop_date;
		$result['change_date'] = $this->change_date;
		$result['change_author_id'] = $this->change_author_id;
		$result['create_date'] = $this->create_date;
		$result['create_author_id'] = $this->create_author_id;
		$result['visible_date'] = $this->visible_date;
		$result['visible_author'] = $this->visible_author_id;
		$result['participants'] = $this->participants;

		if ( $this->template === null )
		{
			$this->loadTemplate();
		}
		$result['template'] = $this->template->flattenToarray();

		if ( $this->seminar === null )
		{
			$this->loadSeminar();
		}
		$result['seminar'] = $this->seminar->flattenToarray();

		return $result;
	}

########################
# STATISCHE FUNKTIONEN #
########################

	/**
	 * Ldt alle relevanten Evaluationsdaten von abgelaufenen Evaluationen eines Semesters gruppiert nach Fakultten.
	 * @param string $semester_id Id des Semesters
	 * @return array Evaluationsdaten
	 * @static
	 * @uses ibit_Evaluation::getEvaluationsFacultyOverview()
	 */
	function getExpiredEvaluationsFacultyOverview( $semester_id = -1 )
	{
		return ibit_Evaluation::getEvaluationsFacultyOverview( $semester_id, 'stop_date < NOW()' );
	}

	/**
	 * Ldt alle relevanten Evaluationsdaten von laufenden Evaluationen eines Semesters gruppiert nach Fakultten.
	 * @param string $semester_id Id des Semesters
	 * @return array Evaluationsdaten
	 * @static
	 * @uses ibit_Evaluation::getEvaluationsFacultyOverview()
	 */
	function getCurrentEvaluationsFacultyOverview( $semester_id = -1 )
	{
		return ibit_Evaluation::getEvaluationsFacultyOverview( $semester_id, 'NOW() BETWEEN start_date AND stop_date' );
	}

	/**
	 * Ldt alle relevanten Evaluationsdaten von zuknftigen Evaluationen eines Semesters gruppiert nach Fakultten.
	 * @param string $semester_id Id des Semesters
	 * @return array Evaluationsdaten
	 * @static
	 * @uses ibit_Evaluation::getEvaluationsFacultyOverview()
	 */
	function getFutureEvaluationsFacultyOverview( $semester_id = -1 )
	{
		return ibit_Evaluation::getEvaluationsFacultyOverview( $semester_id, 'start_date > NOW()' );
	}

	/**
	 * Ldt alle relevanten Evaluationsdaten eines Semesters gruppiert nach Fakultten.
	 * Die Rckgabe wird dabei nach Fakulttsnamen sortiert ausgegeben und hat folgenden Aufbau:
	 *
	 * <pre>
	 *    [
	 *     fakultaet_id     => Id der Fakultt,
	 *     fakultaet_name   => Name der Fakultt,
	 *     seminar_count    => Summe der Seminare,
	 *     institute_count  => Summe der Institute,
	 *     evaluation_count => Summe der Evaluationen,
	 *     participants     => Summe der Teilnehmer an den Evaluationen,
	 *     seminar_members  => Summe der Teilnehmer an den Seminaren
	 *    ]{0..x}
	 * </pre>
	 *
	 * @param string $semester_id Id des Semesters
	 * @param string $additional_where_clause Optionale Einschrnkungen des Queries
	 * @return array Evaluationsdaten
	 * @static
	 * @uses ibit_Helper::getFacultyName()
	 * @uses ibit_EvaluationSeminar::getSeminarMemberCountByFacultyId()
	 */
	function getEvaluationsFacultyOverview( $semester_id = -1, $additional_where_clause = '' )
	{
		$sql = 'SELECT fakultaet_id, COUNT(DISTINCT seminar_id) AS seminar_count, COUNT(DISTINCT institut_id) AS institute_count, COUNT(DISTINCT e.evaluation_id) AS evaluation_count, SUM(u.hidden<>1) AS participants FROM ibit_evaluation e LEFT JOIN ibit_evaluation_user AS u USING(evaluation_id) ###WHERECLAUSE### GROUP BY e.fakultaet_id';
		$where_clause = '';
		if ( $semester_id != -1 )
		{
			if ( !empty( $additional_where_clause ) )
			{
				$where_clause = $additional_where_clause . ' AND ';
			}
			$where_clause .= "semester_id = '{$semester_id}'";
		}

		if ( !empty( $where_clause ) )
		{
			$where_clause = 'WHERE '.$where_clause;
		}
		$sql = str_replace( '###WHERECLAUSE###', $where_clause, $sql );

		$result = DBManager::Get()->query($sql)->fetchAll(PDO::FETCH_ASSOC);
		foreach ($result as &$row)
		{
			$row['fakultaet_name'] = ibit_Helper::getFacultyName($row['fakultaet_id']);
			$row['seminar_members'] = ibit_EvaluationSeminar::getSeminarMemberCountByFacultyId( $row['fakultaet_id'], $semester_id, $additional_where_clause );
		}

		# Ausgabe sortieren nach Fakulttsnamen
		if ( !function_exists('faculty_compare') )
		{
			function faculty_compare( $a, $b )
			{
				return strcmp( $a['fakultaet_name'], $b['fakultaet_name'] );
			}
		}
		usort( $result, 'faculty_compare' );

		return $result;
	}

	/**
	 * Ldt alle relevanten Evaluationsdaten von abgelaufenen Evaluationen einer Fakultt und eines Semesters gruppiert nach Instituten.
	 * @param string $faculty_id Id der Fakultt
	 * @param string $semester_id Id des Semesters
	 * @return array Evaluationsdaten
	 * @static
	 * @uses ibit_Evaluation::getEvaluationsByFacultyId()
	 */
	function getExpiredEvaluationsByFacultyId( $faculty_id, $semester_id = -1 )
	{
		return ibit_Evaluation::getEvaluationsByFacultyId( $faculty_id, $semester_id, 'stop_date < NOW()' );
	}

	/**
	 * Ldt alle relevanten Evaluationsdaten von laufenden Evaluationen einer Fakultt und eines Semesters gruppiert nach Instituten.
	 * @param string $faculty_id Id der Fakultt
	 * @param string $semester_id Id des Semesters
	 * @return array Evaluationsdaten
	 * @static
	 * @uses ibit_Evaluation::getEvaluationsByFacultyId()
	 */
	function getCurrentEvaluationsByFacultyId( $faculty_id, $semester_id = -1 )
	{
		return ibit_Evaluation::getEvaluationsByFacultyId( $faculty_id, $semester_id, 'NOW() BETWEEN start_date AND stop_date' );
	}

	/**
	 * Ldt alle relevanten Evaluationsdaten von zuknftigen Evaluationen einer Fakultt und eines Semesters gruppiert nach Instituten.
	 * @param string $faculty_id Id der Fakultt
	 * @param string $semester_id Id des Semesters
	 * @return array Evaluationsdaten
	 * @static
	 * @uses ibit_Evaluation::getEvaluationsByFacultyId()
	 */
	function getFutureEvaluationsByFacultyId( $faculty_id, $semester_id = -1 )
	{
		return ibit_Evaluation::getEvaluationsByFacultyId( $faculty_id, $semester_id, 'start_date > NOW()' );
	}

	/**
	 * Ldt alle relevanten Evaluationsdaten einer Fakultt und eines Semesters gruppiert nach Instituten.
	 * Die Rckgabe hat dabei folgenden Aufbau:
	 *
	 * <pre>
	 *    [
	 *     institut_id      => Id des Instituts,
	 *     institut_name    => Name des Instituts,
	 *     seminar_count    => Summe der Seminare,
	 *     evaluation_count => Summe der Evaluationen,
	 *     participants     => Summe der Teilnehmer an den Evaluationen,
	 *     seminar_members  => Summe der Teilnehmer an den Seminaren
	 *    ]{0..x}
	 * </pre>
	 *
	 * @param string $faculty_id Id der Fakultt
	 * @param string $semester_id Id des Semesters
	 * @param string $additional_where_clause Optionale Einschrnkungen des Queries
	 * @return array Evaluationsdaten
	 * @static
	 * @uses ibit_EvaluationSeminar::getSeminarMemberCountByInstituteId()
	 */
	function getEvaluationsByFacultyId( $faculty_id, $semester_id = -1, $additional_where_clause = '' )
	{
		$sql = "SELECT e.institut_id, i.name AS institut_name, COUNT(DISTINCT seminar_id) AS seminar_count, COUNT(DISTINCT e.evaluation_id) AS evaluation_count, SUM(u.hidden<>1) AS participants FROM ibit_evaluation AS e INNER JOIN Institute AS i ON (BINARY e.institut_id=BINARY i.Institut_id) LEFT JOIN ibit_evaluation_user AS u ON (e.evaluation_id = u.evaluation_id) WHERE fakultaet_id = '{$faculty_id}' ###WHERECLAUSE### GROUP BY institut_id ORDER BY institut_name";

		$where_clause = '';
		if ( $semester_id != -1 )
		{
			if ( !empty( $additional_where_clause ) )
			{
				$where_clause = $additional_where_clause . ' AND ';
			}
			$where_clause .= "semester_id = '{$semester_id}'";
		}

		if ( !empty( $where_clause ) )
		{
			$where_clause = 'AND '.$where_clause;
		}

		$sql = str_replace( '###WHERECLAUSE###', $where_clause, $sql );

		$result = DBManager::Get()->query($sql)->fetchAll(PDO::FETCH_ASSOC);
		foreach ($result as &$row)
		{
			$row['seminar_members'] = ibit_EvaluationSeminar::getSeminarMemberCountByInstituteId($row['institut_id'], $semester_id, $additional_where_clause);
		}
		return $result;
	}

	/**
	 * Ldt alle relevanten Evaluationsdaten von abgelaufenen Evaluationen eines Instituts und eines Semesters.
	 * @param string $institute_ids Ids der Institute
	 * @param int $skip Anzahl der Evaluationen, die bersprungen werden sollen
	 * @param int $limit Anzahl der Evaluationen, die ausgelesen werden sollen
	 * @param int $count Anzahl der ausgelesenen Evaluationen (Output-Parameter)
	 * @param string $semester_id Id des Semesters
	 * @return array Evaluationsdaten
	 * @static
	 * @uses ibit_Evaluation::getEvaluationsByInstituteId()
	 */
	function getExpiredEvaluationsByInstituteId( $institute_ids, &$skip, &$limit, &$count, $semester_id = -1, $id_only=false )
	{
		return ibit_Evaluation::getEvaluationsByInstituteId( $institute_ids, $skip, $limit, $count, $semester_id, 'stop_date < NOW()', $id_only );
	}

	/**
	 * Ldt alle relevanten Evaluationsdaten von laufenden Evaluationen eines Instituts und eines Semesters.
	 * @param string $institute_ids Ids der Institute
	 * @param int $skip Anzahl der Evaluationen, die bersprungen werden sollen
	 * @param int $limit Anzahl der Evaluationen, die ausgelesen werden sollen
	 * @param int $count Anzahl der ausgelesenen Evaluationen (Output-Parameter)
	 * @param string $semester_id Id des Semesters
	 * @return array Evaluationsdaten
	 * @static
	 * @uses ibit_Evaluation::getEvaluationsByInstituteId()
	 */
	function getCurrentEvaluationsByInstituteId( $institute_ids, &$skip, &$limit, &$count, $semester_id = -1, $id_only=false )
	{
		return ibit_Evaluation::getEvaluationsByInstituteId( $institute_ids, $skip, $limit, $count, $semester_id, 'NOW() BETWEEN start_date AND stop_date', $id_only );
	}

	/**
	 * Ldt alle relevanten Evaluationsdaten von zuknftigen Evaluationen eines Instituts und eines Semesters.
	 * @param string $institute_ids Ids der Institute
	 * @param int $skip Anzahl der Evaluationen, die bersprungen werden sollen
	 * @param int $limit Anzahl der Evaluationen, die ausgelesen werden sollen
	 * @param int $count Anzahl der ausgelesenen Evaluationen (Output-Parameter)
	 * @param string $semester_id Id des Semesters
	 * @return array Evaluationsdaten
	 * @static
	 * @uses ibit_Evaluation::getEvaluationsByInstituteId()
	 */
	function getFutureEvaluationsByInstituteId( $institute_ids, &$skip, &$limit, &$count, $semester_id = -1, $id_only=false )
	{
		return ibit_Evaluation::getEvaluationsByInstituteId( $institute_ids, $skip, $limit, $count, $semester_id, 'start_date > NOW()', $id_only );
	}

	/**
	 * Ldt alle relevanten Evaluationsdaten eines Instituts und eines Semesters.
	 * Die Rckgabe hat dabei folgenden Aufbau:
	 *
	 * <pre>
	 *    [
	 *     evaluation_id    => Id der Evaluation,
	 *     evaluation_title => Name der Evaluation,
	 *     seminar_id       => Id des Seminars,
	 *     seminar_title    => Name des Seminars
	 *     seminar_members  => Summe der Teilnehmer an dem Seminar,
	 *     template_id      => Id der Vorlage der Evaluation,
	 *     status           => Status der Evaluation,
	 *     start_date       => Startdatum der Evaluation,
	 *     stop_date        => Stopdatum der Evaluation,
	 *     participants     => Anzahl der Teilnehmer an der Evaluation
	 *    ]{0..x}
	 * </pre>
	 *
	 * @param string $institute_ids Ids der Institute
	 * @param int $skip Anzahl der Evaluationen, die bersprungen werden sollen
	 * @param int $limit Anzahl der Evaluationen, die ausgelesen werden sollen
	 * @param int $count Anzahl der ausgelesenen Evaluationen (Output-Parameter)
	 * @param string $semester_id Id des Semesters
	 * @param string $additional_where_clause Optionale Einschrnkungen des Queries
	 * @return array
	 * @static
	 * @uses ibit_EvaluationSeminar
	 * @uses ibit_Helper::getSeminarName()
	 */
	function getEvaluationsByInstituteId( $institute_ids, &$skip, &$limit, &$count, $semester_id, $additional_where_clause, $id_only )
	{
		$sql = "SELECT e.seminar_id, e.evaluation_template_id AS template_id, e.evaluation_id, title AS evaluation_title, UNIX_TIMESTAMP( start_date) AS start_date, UNIX_TIMESTAMP(stop_date) AS stop_date, CASE WHEN NOW() > stop_date THEN 'expired' WHEN NOW() < start_date THEN 'future' ELSE 'current' END AS status, SUM(u.hidden<>1) AS participants, s.Name AS seminar_title FROM ibit_evaluation AS e INNER JOIN ibit_evaluation_templates AS t USING (evaluation_template_id) LEFT JOIN ibit_evaluation_user AS u ON (u.evaluation_id=e.evaluation_id) LEFT JOIN seminare s ON (s.Seminar_id=e.seminar_id) WHERE e.institut_id IN (###IN_CLAUSE###) ###WHERECLAUSE### GROUP BY evaluation_id ORDER BY s.Name ASC, e.evaluation_template_id, e.seminar_id, start_date, stop_date";

		if($id_only == true)
		{
			$sql = "SELECT evaluation_id FROM ibit_evaluation WHERE institut_id IN (###IN_CLAUSE###) ###WHERECLAUSE### ";
		}

		if ( !is_array( $institute_ids ) )
		{
			$institute_ids = array( $institute_ids );
		}

		if ( $skip !== null and $limit !== null )
		{
			$sql .= " LIMIT {$skip}, {$limit}";
		}

		if ( $semester_id != -1 )
		{
			if ( !empty( $additional_where_clause ) )
			{
				$additional_where_clause .= ' AND ';
			}
			$additional_where_clause .= "semester_id = '{$semester_id}'";
		}
		$additional_where_clause = 'AND '.$additional_where_clause;
		$in_clause = "'".implode("','", $institute_ids )."'";

		$sql_count = 'SELECT COUNT(*) AS evaluation_count FROM ibit_evaluation WHERE institut_id IN (###IN_CLAUSE###) ###WHERECLAUSE###';
		$sql_count = str_replace( '###IN_CLAUSE###', $in_clause, $sql_count );
		$sql_count = str_replace( '###WHERECLAUSE###', $additional_where_clause, $sql_count );
		$sql = str_replace( '###IN_CLAUSE###', $in_clause, $sql );
		$sql = str_replace( '###WHERECLAUSE###', $additional_where_clause, $sql );

		$result = DBManager::Get()->query($sql)->fetchAll(PDO::FETCH_ASSOC);
		foreach ($result as &$row)
		{
			if (isset($row['seminar_id']))
			{
				$seminar = new ibit_EvaluationSeminar($row['seminar_id']);
				$row['seminar_members'] = $seminar->getMembers();
			}
		}

		$count = DBManager::Get()->query($sql_count)->fetchColumn();
		if ( $skip != null and $limit != null )
		{
			$skip = max ( $skip - $count, 0 );
			$limit = $limit - count( $result );
		}

		return $result;
	}

	/**
	 * Liefert Infos ber die Evaluationen, die einem User zugeordnet sind, zurck.
	 * Es werden nur laufende und beendete Evaluationen ausgelesen. In der Studentensicht werden bei den beendeten
	 * Evaluationen nur die Evaluationen ausgegeben, an denen der Student teilgenommen hat und die laufendenen Evaluationen
	 * werden unterteilt in "teilgenommen" und "nicht teilgenommen". Die Rckgabe wird jeweils nach Status (und
	 * Teilnahmestatus), Semester und Fakultt aufgeteilt ausgegeben.
	 *
	 * Die Rckgabe fr den Dozenten hat folgenden Aufbau:
	 *
	 * <pre>
	 *    [
	 *     running => [
	 *       <semester_name> => [
	 *          <fakultaets_id> => [
	 *             name        => Name der Fakultt,
	 *             evaluations => EVALUATION {0..x}
	 *                             ]{0..x}
	 *                          ]{0..x}
	 *                ]{0..x},
	 *     expired => [
	 *       <semester_name> => [
	 *          <fakultaets_id> => [
	 *             name        => Name der Fakultt,
	 *             evaluations => EVALUATION {0..x}
	 *                             ]{0..x}
	 *                          ]{0..x}
	 *                ]{0..x}
	 *    ]{0..x}
	 * </pre>
	 *
	 * Fr den Studenten hat die Rckgabe folgenden Aufbau:
	 *
	 * <pre>
	 *    [
	 *     running => [
	 *       participated     => [
	 *         <semester_name> => [
	 *            <fakultaets_id> => [
	 *               name        => Name der Fakultt,
	 *               evaluations => EVALUATION {0..x}
	 *                               ]{0..x}
	 *                            ]{0..x}
	 *                           ]{0..x}
	 *                ],
	 *       not_participated => [
	 *         <semester_name> => [
	 *            <fakultaets_id> => [
	 *               name        => Name der Fakultt,
	 *               evaluations => EVALUATION {0..x}
	 *                               ]{0..x}
	 *                            ]{0..x}
	 *                           ]{0..x}
     *                ],
     *     expired => [
	 *       <semester_name> => [
	 *          <fakultaets_id> => [
	 *             name        => Name der Fakultt,
	 *             evaluations => EVALUATION {0..x}
	 *                             ]{0..x}
	 *                          ]{0..x}
	 *                ]{0..x}
	 *    ]{0..x}
	 * </pre>
	 *
	 * EVALUATION wiederum hat dabei folgenden Aufbau:
	 *
	 * <pre>
	 *    [
	 *     evaluation_id   => Id der Evaluation,
	 *     fakultaet_id    => Id der Fakultt,
	 *     semester        => Name des Semesters,
	 *     seminar_id      => Id des Seminars,
	 *     seminar_name    => Name des Seminars,
	 *     seminar_members => Teilnehmer des Seminars,
	 *     participated    => Gibt an, ob der User an der Evaluation teilgenommen hat,
	 *     participants    => Anzahl der Teilnehmer an der Evaluation,
	 *     start_date      => Startdatum der Evaluation,
	 *     stop_date       => Enddatum der Evaluation,
	 *     visible         => Gibt an, ob die Evaluation freigeschaltet ist,
	 *     running         => Gibt an, ob die Evaluation luft oder beendet ist
	 *    ]
	 * </pre>
	 *
	 * @param string $user_id Id des Users
	 * @param boolean $dozent Gibt an, ob der User als Dozent oder Student betrachtet werden soll
	 * @return array Informationen ber die Evaluationen in der Form
	 * @static
	 * @uses ibit_Helper::getSemester()
	 * @uses ibit_Helper::getFacultyName()
	 * @uses ibit_EvaluationSeminar
	 */
	function getEvaluationDataByUserId( $user_id, $dozent = false )
	{
		$sql = "SELECT u.Seminar_id, CONCAT(VeranstaltungsNummer,' - ', Name) AS Name FROM seminar_user AS u INNER JOIN seminare USING(Seminar_id) WHERE user_id = '{$user_id}'";
		if ( $dozent )
		{
			$sql .= " AND u.status IN ('dozent', 'admin')";
		}
		else
		{
			$sql .= " AND u.status NOT IN ('dozent', 'admin')";
		}

		$rows = DBManager::Get()->query($sql)->fetchAll(PDO::FETCH_ASSOC);

		$seminar_names = array();
		foreach ($rows as $row)
		{
			$seminar_names[ $row['Seminar_id'] ] = $row['Name'];
		}
		$seminar_ids = array_keys($seminar_names);

		if ( empty($seminar_ids) )
		{
			return false;
		}

		$semesters = array();
		foreach (ibit_Helper::getSemester() as $semester)
		{
			$semesters[ $semester['id'] ] = $semester['name'];
		}

		$sql = "SELECT e.evaluation_id, seminar_id, semester_id, IF(start_date>NOW(), 1, 0) AS future, IF(stop_date>NOW() AND start_date<NOW(), 1, 0) AS running, IF(user_id IS NULL, 0, 1) AS participated, UNIX_TIMESTAMP(start_date) AS start_date, UNIX_TIMESTAMP(stop_date) AS stop_date, IF (visible_date IS NULL, 0, 1) AS visible, fakultaet_id FROM ibit_evaluation AS e LEFT JOIN ibit_evaluation_user AS u ON (user_id='{$user_id}' AND u.evaluation_id=e.evaluation_id) WHERE seminar_id IN ('".implode("','", $seminar_ids)."') ORDER BY semester_id ASC";

		$rows = DBManager::Get()->query($sql)->fetchAll(PDO::FETCH_ASSOC);

		$ids = array();
		foreach ($rows as &$row)
		{
			$seminar = new ibit_EvaluationSeminar($row['seminar_id']);
			$row['seminar_members'] = $seminar->getMembers();

			$row['semester'] = $semesters[ $row['semester_id'] ];
			$row['seminar_name'] = $seminar_names[ $row['seminar_id'] ];

			array_push($ids, $row['evaluation_id']);
		}

		$participants = array();
		if (!empty($ids))
		{
			$statement = DBManager::Get()->query("SELECT evaluation_id, SUM(hidden<>1) AS participants FROM ibit_evaluation_user WHERE evaluation_id IN ('".implode("','", $ids)."') GROUP BY evaluation_id");
			while ($row = $statement->fetch(PDO::FETCH_ASSOC))
			{
				$participants[ $row['evaluation_id'] ] = $row['participants'];
			}
		}

		if( $dozent )
		{
			$result = array( 'expired'=>array(), 'running'=>array(), 'future'=>array() );
		}
		else
		{
			$result = array( 'expired'=>array(), 'running'=>array('participated'=>array(), 'not_participated'=>array()) );
		}

		foreach ( $rows as $row )
		{
			$row['participants'] = empty($participants[ $row['evaluation_id'] ]) ? 0 : $participants[ $row['evaluation_id'] ];

			$time_index = $row['running']
				? 'running'
				: ($row['future']
					? 'future'
					: 'expired');

			#dozentenansicht
			if ($dozent or ($time_index=='expired' and $row['participated']))
			{
				if ( empty( $result[$time_index][ $row['semester'] ] ) )
				{
					$result[$time_index][ $row['semester'] ] = array();
				}
				if ( empty( $result[$time_index][ $row['semester'] ][ $row['fakultaet_id'] ] ) )
				{
					$result[$time_index][ $row['semester'] ][ $row['fakultaet_id'] ] = array(
						'name' => ibit_Helper::getFacultyName( $row['fakultaet_id'] ),
						'evaluations' => array()
					);
				}
				array_push($result[$time_index][ $row['semester'] ][ $row['fakultaet_id'] ]['evaluations'], $row);
			}
			# studentenansicht
			elseif (!$dozent and $time_index=='running')
			{
				$part_index = $row['participated'] ? 'participated' : 'not_participated';
				if ( empty( $result[$time_index][$part_index][ $row['semester'] ] ) )
				{
					$result[$time_index][$part_index][ $row['semester'] ] = array();
				}
				if ( empty( $result[$time_index][$part_index][ $row['semester'] ][ $row['fakultaet_id'] ] ) )
				{
					$result[$time_index][$part_index][ $row['semester'] ][ $row['fakultaet_id'] ] = array(
						'name' => ibit_Helper::getFacultyName( $row['fakultaet_id'] ),
						'evaluations' => array()
					);
				}
				array_push($result[$time_index][$part_index][ $row['semester'] ][ $row['fakultaet_id'] ]['evaluations'], $row);
			}
		}

		return $result;
	}

	/**
	 * Prft, ob ein User fr eine Evaluation zugelassen ist.
	 * @param string $user_id Id des Users
	 * @param int $evaluation_id Id der Evaluation
	 * @return boolean Gibt an, ob der User fr die Evaluation zugelassen ist.
	 * @static
	 */
	function userValidForEvaluation( $user_id, $evaluation_id )
	{
		$seminar_id = DBManager::Get()->query("SELECT seminar_id FROM ibit_evaluation WHERE evaluation_id = '{$evaluation_id}'")->fetchColumn();
		if (!$seminar_id)
		{
			return false;
		}
		return DBManager::Get()->query("SELECT 1 FROM seminar_user WHERE Seminar_id='{$seminar_id}' AND user_id='{$user_id}'")->fetchColumn();
	}

	/**
	 * Prft, ob ein User an einer Evaluation teilgenommen hat.
	 * @param string $user_id Id des Users
	 * @param int $evaluation_id Id der Evaluation
	 * @return boolean Gibt an, ob ein User an einer Evaluation teilgenommen hat.
	 * @static
	 */
	function userParticipatedInEvaluation( $user_id, $evaluation_id )
	{
		return DBManager::Get()->query("SELECT 1 FROM ibit_evaluation_user WHERE user_id='{$user_id}' AND evaluation_id='{$evaluation_id}'")->fetchColumn();
	}

	/**
	 * Ldt Evaluations-Id fr den Export
	 * @param string $semester_id Id des Semesters
	 * @param int $min_users Minimale Anzahl an teilnehmenden Studenten
	 * @param boolean $visible Gibt an, ob nur freigeschaltete Evaluationen ausgegeben werden sollen
	 * @return array array mit den Evaluations-IDs
	 * @static
	 */
	function getEvaluationExportIDs($semester_id, $min_users, $visible=true)
	{
		$evaluation_ids = array();

		# Evaluationen auslesen
		$sql_load_evaluations = "SELECT e.evaluation_id FROM ibit_evaluation AS e LEFT JOIN ibit_evaluation_user AS u USING( evaluation_id ) WHERE ##additional_where_clause## stop_date < NOW() AND semester_id = '{$semester_id}' GROUP BY e.evaluation_id HAVING SUM(u.hidden<>1)>{$min_users}";

		$additional_where_clause = '';
		if ($visible)
		{
			$additional_where_clause = 'visible_date IS NOT NULL AND visible_date <= NOW() AND visible_author_id IS NOT NULL AND';
		}
		$sql_load_evaluations = str_replace( '##additional_where_clause##', $additional_where_clause, $sql_load_evaluations );

		return DBManager::Get()->query($sql_load_evaluations)->fetchAll(PDO::FETCH_COLUMN);
	}

	/**
	 * Ldt eine Evaluation inklusive aller Antworten.
	 * @param string $evaluation_id Id der Evaluation
	 * @return array Evaluationsdaten
	 * @static
	 */
	function getEvaluationAnswers($evaluation_id)
	{
		$evaluation = DBManager::Get()->query("SELECT e.evaluation_id AS id, seminar_id, institut_id, fakultaet_id, SUM(u.hidden<>1) AS participants FROM ibit_evaluation AS e LEFT JOIN ibit_evaluation_user AS u USING( evaluation_id ) WHERE e.evaluation_id = '{$evaluation_id}' GROUP BY e.evaluation_id")->fetchColumn();

		if (!$evaluation)
		{
			return false;
		}

		$evaluation['groups'] = $group_ids = $question_ids = array();

		# Struktur laden
		$rows = DBManager::Get()->query("SELECT evaluation_group_template_id AS group_id, evaluation_question_template_id AS question_id, group_position, question_position FROM ibit_evaluation_links AS l INNER JOIN ibit_evaluation USING( evaluation_template_id ) WHERE evaluation_id='{$evaluation_id}' ORDER BY group_position ASC, question_position ASC")->fetchAll(PDO::FETCH_ASSOC);
		foreach ($rows as $row)
		{
			if (empty($evaluation['groups'][ $row['group_id'] ]))
			{
				$evaluation['groups'][ $row['group_id'] ] = array(
					'group_id' => $row['group_id'],
					'position' => $row['group_position'],
					'questions' => array()
				);
				array_push( $group_ids, $row['group_id'] );
			}
			if (empty($evaluation['groups'][ $row['group_id'] ]['questions'][ $row['question_id'] ]))
			{
				$evaluation['groups'][ $row['group_id'] ]['questions'][ $row['question_id'] ] = array(
					'question_id' => $row['question_id'],
					'position' => $row['question_position'],
					'answers' => array()
				);
				array_push( $question_ids, $row['question_id'] );
			}
		}

		# Antworten auslesen
		$rows = DBManager::Get()->query("SELECT evaluation_group_template_id AS group_id, evaluation_question_template_id AS question_id, evaluation_answer_template_id AS answer_id, TRIM(value) AS value, COUNT(*) AS anzahl FROM ibit_evaluation_answers WHERE evaluation_id='{$evaluation_id}' GROUP BY evaluation_id, evaluation_group_template_id, evaluation_question_template_id, evaluation_answer_template_id, LOWER(TRIM(value))")->fetchAll(PDO::FETCH_ASSOC);
		foreach ($rows as $row)
		{
			array_push($evaluation['groups'][ $row['group_id'] ]['questions'][ $row['question_id'] ]['answers'], array(
				'answer_id' => $row['answer_id'],
				'user_value' => $row['value'],
				'count' => $row['anzahl']
			));
			array_push($answer_ids, $rows['answer_id']);
		}

		# Gruppendaten auslesen
		$group_data = $question_data = $answer_data = array();

		$rows = DBManager::Get()->query("SELECT evaluation_group_template_id AS group_id, title, description FROM ibit_evaluation_group_templates WHERE evaluation_group_template_id IN ('".implode("','", array_unique($group_ids))."')")->fetchAll(PDO::FETCH_ASSOC);
		foreach ($rows as $row)
		{
			$group_data[ $row['group_id'] ] = array(
				'title' => $row['title'],
				'description' => $row['description']
			);
		}

		# Fragendaten auslesen
		$rows = DBManager::Get()->query("SELECT evaluation_question_template_id AS question_id, q.title AS title, q.text AS text, t.title AS answer_title, t.description AS answer_description, t.html_type AS html_type FROM ibit_evaluation_question_templates AS q LEFT JOIN ibit_evaluation_answer_types AS t USING( evaluation_answer_type_id ) WHERE evaluation_question_template_id IN ('".implode("','", array_unique($question_ids))."')")->fetchAll(PDO::FETCH_ASSOC);
		foreach ($rows as $row)
		{
			$question_data[ $row['question_id']] = array(
				'title' => $row['title'],
				'text' => $row['text'],
				'answer_title' => $row['answer_title'],
				'answer_description' => $row['answer_description'],
				'html_type' => $row['html_type']
			);
		}

		# Antwortdaten auslesen
		$rows = DBManager::Get()->query("SELECT evaluation_answer_template_id AS answer_id, text, position, value FROM ibit_evaluation_answer_templates WHERE evaluation_answer_template_id IN ('".implode("','", array_unique($answer_ids))."')")->fetchAll(PDO::FETCH_ASSOC);
		foreach ($rows as $row)
		{
			$answer_data[ $row['answer_id'] ] = array(
				'text' => $row['text'],
				'position' => $row['position'],
				'value' => $row['value']
			);
		}

		# Daten ins array fllen
		foreach ($evaluation['groups'] as $group_idx=>$group)
		{
			$group['title'] = $group_data[$group_idx]['title'];
			$group['description'] = $group_data[$group_idx]['description'];
			foreach ($group['questions'] as $question_idx=>$question)
			{
				$question['title'] = $question_data[ $question_idx ]['title'];
				$question['text'] = $question_data[ $question_idx ]['text'];
				$question['answer_title'] = $question_data[ $question_idx ]['answer_title'];
				$question['answer_description'] = $question_data[ $question_idx ]['answer_description'];
				$question['html_type'] = $question_data[ $question_idx ]['html_type'];
				foreach ($question['answers'] as $answer_idx=>$answer)
				{
					$answer['text'] = $answer_data[ $answer['answer_id'] ]['text'];
					$answer['position'] = $answer_data[ $answer['answer_id'] ]['position'];
					$answer['value'] = $answer_data[ $answer['answer_id'] ]['value'];
					$question['answers'][ $answer_idx ] = $answer;
				}
				$group['questions'][ $question_idx ] = $question;
			}
			$evaluation['groups'][ $group_idx ] = $group;
		}

		# Daten zurckgeben
		return $evaluation;
	}

	/**
	 * Liefert die Anzahl der Teilnehmer an einer Menge von Evaluationen zurck.
	 * @param array $evaluation_ids Ids der Evaluationen
	 * @return array Anzahl der Teilnehmer gruppiert nach Id der jeweiligen Evaluation
	 * @static
	 */
	function getEvaluationParticipants( $evaluation_ids )
	{
		$result = array();

		$rows = DBManager::Get()->query("SELECT evaluation_id, SUM(hidden<>1) AS participants FROM ibit_evaluation_user WHERE evaluation_id IN (".implode(',', (array)$evaluation_ids).") GROUP BY evaluation_id")->fetchAll(PDO::FETCH_ASSOC);
		foreach ($rows as $row)
		{
			$result[ $row['evaluation_id'] ] = $row['participants'];
		}

		return $return;
	}

	/**
	 * Liest diverse statistische Informationen ber die Evaluationen eines Semesters aus.
	 * Die Rckgabe sieht dabei folgendermassen aus:
	 *
	 * <pre>
	 *    [
	 *     evaluation_count  => Anzahl der Evaluationen,
	 *     visible_count     => Anzahl der freigeschalteten Evaluationen,
	 *     not_visible_count => Anzahl der nicht freigeschalteten Evaluationen,
	 *     participant_count => Gesamtzahl der Teilnehmer an den Evaluationen
	 *    ]
	 * </pre>
	 */
	function getEvaluationStatisticsForSemester( $semester_id )
	{
		$return = array();

		# Anzahl der Evaluation, der freigeschalteten und der nicht freigeschalteten Evaluationen des Semesters ermitteln
		$rows = DBManager::Get()->query("SELECT COUNT(*) AS evaluation_count, SUM( visible_date IS NOT NULL ) AS visible_count, fakultaet_id, institut_id FROM ibit_evaluation WHERE semester_id = '{$semester_id}' GROUP BY fakultaet_id, institut_id WITH ROLLUP")->fetchAll(PDO::FETCH_ASSOC);
		foreach ($rows as $row)
		{
			if (!isset($return[ $row['fakultaet_id'] ]))
			{
				$return[ $row['fakultaet_id'] ] = array();
			}
			if (!isset($return[ $row['fakultaet_id'] ][ $row['institut_id'] ]) )
			{
				$return[ $row['fakultaet_id'] ][ $row['institut_id'] ] = array();
			}
			$return[ $row['fakultaet_id'] ][ $row['institut_id'] ]['evaluation_count'] = $row['evaluation_count'];
			$return[ $row['fakultaet_id'] ][ $row['institut_id'] ]['visible_count'] = $row['visible_count'];
			$return[ $row['fakultaet_id'] ][ $row['institut_id'] ]['not_visible_count'] = $row['evaluation_count'] - $row['visible_count'];
		}

		# Anzahl der Teilnehmer an den Evaluation des Semester ermitteln
		DBManager::Get()->query("SELECT SUM(u.hidden<>1) AS participant_count, COUNT( DISTINCT user_id ) AS participant_unique_count, fakultaet_id, institut_id FROM ibit_evaluation_user AS u INNER JOIN ibit_evaluation AS e USING(evaluation_id) WHERE e.semester_id = '{$semester_id}' GROUP BY fakultaet_id, institut_id WITH ROLLUP")->fetchAll(PDO::FETCH_ASSOC);
		foreach ($rows as $row)
		{
			if (!isset($return[ $row['fakultaet_id'] ]))
			{
				$return[ $row['fakultaet_id'] ] = array();
			}
			if (!isset($return[ $row['fakultaet_id'] ][ $row['institut_id'] ]) )
			{
				$return[ $row['fakultaet_id'] ][ $row['institut_id'] ] = array();
			}
			$return[ $row['fakultaet_id'] ][ $row['institut_id'] ]['participant_count'] = $row['participant_count'];
			$return[ $row['fakultaet_id'] ][ $row['institut_id'] ]['participant_count_unique'] = $row['participant_unique_count'];
		}

		# Anzahl der Teilnehmer an den Seminaren der Evaluationen ermitteln
		$rows = DBManager::Get()->query("SELECT COUNT(*) AS seminar_members, COUNT( DISTINCT user_id ) AS seminar_members_unique, fakultaet_id, institut_id FROM ibit_evaluation AS e LEFT JOIN seminar_user AS u ON (e.seminar_id=u.seminar_id) WHERE e.semester_id = '{$semester_id}' AND u.status NOT IN('dozent', 'tutor', 'admin') GROUP BY fakultaet_id, institut_id WITH ROLLUP")->fetchAll(PDO::FETCH_ASSOC);
		foreach ($rows as $row)
		{
			if (!isset($return[ $row['fakultaet_id'] ]))
			{
				$return[ $row['fakultaet_id'] ] = array();
			}
			if (!isset($return[ $row['fakultaet_id'] ][ $row['institut_id'] ]) )
			{
				$return[ $row['fakultaet_id'] ][ $row['institut_id'] ] = array();
			}
			$return[ $row['fakultaet_id'] ][ $row['institut_id'] ]['seminar_members'] = $row['seminar_members'];
			$return[ $row['fakultaet_id'] ][ $row['institut_id'] ]['seminar_members_unique'] = $row['seminar_members_unique'];
		}

		foreach ($return as $faculty_id=>$faculty_data)
		{
			foreach ($faculty_data as $institute_id=>$institute_data)
			{
				if (empty($institute_data['participant_count']))
				{
					$return[$faculty_id][$institute_id]['participant_count'] = 0;
					$return[$faculty_id][$institute_id]['participant_count_unique'] = 0;
				}
				if (empty($institute_data['seminar_members']))
				{
					$return[$faculty_id][$institute_id]['seminar_members'] = 0;
					$return[$faculty_id][$institute_id]['seminar_members_unique'] = 0;
				}
			}
		}

		return $return;
	}

	/**
	 * Liefert Infos einer Evaluation zu einer Veranstaltung zurck
	 * @param string $seminar_id Id der Veranstaltung
	 * @return array Infos zur Evaluation (Teilnehmer, ID, Freischaltung)
	 */
	function getEvaluationBySeminar( $seminar_id )
	{
		$return = DBManager::Get()->query("SELECT SUM(u.hidden<>1) AS participants, visible_date AS visible, e.evaluation_id FROM ibit_evaluation_user AS u INNER JOIN ibit_evaluation AS e USING(evaluation_id) WHERE e.seminar_id = '{$seminar_id}' GROUP BY e.evaluation_id")->fetchColumn();

		return empty($return)
			? array('status'=>_('keine Evaluationen vorhanden'))
			: $return;
	}

	/**
	 * Not documented yet
	 * @static
	 */
	function getFacultyForSeminar($seminar_id)
	{
		return DBManager::Get()->query("SELECT fakultaet_id FROM ibit_evaluation WHERE seminar_id='{$seminar_id}'")->fetchColumn();
	}
}
?>