<?php
/**
 * classes/ibit_EvaluationQuestion.class.php
 *
 * @author  Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @author  Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author  Ivn Marcos <ivan.marcos@gmx.net>
 * @author  Nico Mueller <nico.mueller@uni-oldenburg.de>
 * @package IBIT_Evaluation
 * @copyright IBIT 2006-2008
 * @version 2.0.15
 */

/**
 * Die Reprsentation einer Evaluationsfrage.
 *
 * Diese Klasse stellt eine Reprsentation der Daten einer Frage einer Evaluation dar.
 *
 * @author     Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author     Michael Riehemann <michael.riehemann@mail.uni-oldenburg.de>
 * @author     Ivn Marcos <ivan.marcos@gmx.net>
 * @package    IBIT_Evaluation
 * @subpackage Structure
 * @copyright  Copyright 2006, 2007 IBIT
 * @version    1.1
 */
class ibit_EvaluationQuestion
{
	/**
	 * Die Id der Evaluationsfrage
	 * @var string $evaluation_question_template_id
	 */
	var $evaluation_question_template_id = null;

	/**
	 * Array, das die Antworten der Evaluationsfrage enthlt
	 * @var array $answers
	 */
	var $answers;

	/**
	 * Der Titel der Evaluationsfrage
	 * @var string $title
	 */
	var $title;

	/**
	 * Der Text der Evaluationsfrage
	 * @var string $text
	 */
	var $text;

	/**
	 * Der Geschtzt-Status der Evaluationsfrage
	 * @var int $protected
	 */
	var $protected;

	/**
	 * Die Id des zur Evaluationsfrage gehrenden Antworttypen
	 * @var int $answer_type_id
	 */
	var $answer_type_id;

	/**
	 * Das Datum der letzten nderung der Evaluationsfrage
	 * @var int $change_date
	 */
	var $change_date;

	/**
	 * Die Position der Evaluationsfrage innerhalb der Gruppe
	 * @var int $position
	 */
	var $position;

	/**
	 * Der HTML-Typ der Evaluationsfrage
	 * @var string $html_type
	 */
	var $html_type;

	/**
	 * Der Konstruktor der Klasse.
	 * @param int $id Id der Evaluationsfrage
	 */
	function ibit_EvaluationQuestion( $id = null )
	{
		if ( $id !== null )
		{
			$this->load( $id );
		}
	}

	/**
	 * Ldt die Daten der Evaluationsfrage mit der bergebenen Id in dieses Objekt.
	 * @param string $eval_question_template_id Id der Evaluationsfrage
	 * @return boolean Gibt an, ob das Laden erfolgreich war.
	 */
	function load( $eval_question_template_id )
	{
		$this->evaluation_question_template_id = $eval_question_template_id;

		$data = DBManager::Get()->query("SELECT q.title, `text`, change_date, protected, q.evaluation_answer_type_id, html_type FROM ibit_evaluation_question_templates AS q LEFT JOIN ibit_evaluation_answer_types USING(evaluation_answer_type_id) WHERE evaluation_question_template_id = '{$this->evaluation_question_template_id}'")->fetch(PDO::FETCH_ASSOC);

		if (!$data)
		{
			return false;
		}

		$this->title = $data['title'];
		$this->text = $data['text'];
		$this->protected = $data['protected'];
		$this->change_date = $data['change_date'];
		$this->answer_type_id = $data['evaluation_answer_type_id'];
		$this->html_type = $data['html_type'];

		return true;
	}

	/**
	 * Speichert die Daten der Evaluationsfrage in der Datenbank.
	 */
	function save()
	{
		$result = DBManager::Get()->exec("INSERT INTO ibit_evaluation_question_templates (evaluation_question_template_id, title, `text`, change_date, protected, evaluation_answer_type_id) VALUES ('{$this->evaluation_question_template_id}', '{$this->title}', '{$this->text}', NOW(), '{$this->protected}', '{$this->answer_type_id}') ON DUPLICATE KEY UPDATE title=VALUES(title), `text`=VALUES(`text`), change_date=VALUES(change_date), protected=VALUES(protected), evaluation_answer_type_id=VALUES(evaluation_answer_type_id)");
		if ($result and $this->evaluation_question_template_id === null)
		{
			$this->evaluation_question_template_id = DBManager::Get()->lastInsertId();
		}
	}

	/**
	 * Ldt die Antworten der Evaluationsfrage.
	 * @return boolean Gibt an, ob das Laden erfolgreich war.
	 * @uses ibit_EvaluationAnswer
	 */
	function loadAnswers()
	{
		$this->answers = array();

		$ids = DBManager::Get()->query("SELECT evaluation_answer_template_id FROM ibit_evaluation_answer_templates WHERE evaluation_answer_type_id = '{$this->answer_type_id}' ORDER BY position ASC")->fetchAll(PDO::FETCH_COLUMN);
		foreach ($ids as $id)
		{
			array_push($this->answers, new ibit_EvaluationAnswer($id));
		}

		return !empty($ids);
	}

	/**
	 * Setzt die Id der Evaluationsfrage.
	 * @param $evaluation_question_template_id Id der Evaluationsfrage
	 */
	function setEvaluationQuestionId( $evaluation_question_template_id )
	{
		$this->evaluation_question_template_id = $evaluation_question_template_id;
	}

	/**
	 * Liefert die Id der Evaluationsfrage zurck.
	 * @return int Id der Evaluationsfrage
	 */
	function getEvaluationQuestionId()
	{
		return $this->evaluation_question_template_id;
	}

	/**
	 * Liefert die Id der Evaluationsfrage zurck.
	 * @return int Id der Evaluationsfrage
	 */
	function getId()
	{
		return $this->getEvaluationQuestionId();
	}

	/**
	 * Setzt den Titel der Evaluationsfrage.
	 * @param string $title Titel der Evaluationsfrage
	 */
	function setTitle( $title )
	{
		$this->title = $title;
	}

	/**
	 * Liefert den Titel der Evaluationsfrage zurck.
	 * @return string Titel der Evaluationsfrage
	 */
	function getTitle()
	{
		return $this->title;
	}

	/**
	 * Setzt den Text der Evaluationsfrage.
	 * @param string $text Text der Evaluationsfrage
	 */
	function setText( $text )
	{
		$this->text = $text;
	}

	/**
	 * Liefert den Text der Evaluationsfrage zurck.
	 * @return string Text der Evaluationsfrage
	 */
	function getText()
	{
		return $this->text;
	}

	/**
	 * Liefert den Geschtzt-Status der Evaluationsfrage zurck
	 * @return int Geschtzt-Status der Evaluationsfrage
	 */
	function getProtected()
	{
		return $this->protected;
	}

	/**
	 * Setzt den Geschtzt-Status der Evaluationsfrage.
	 * @param int $protected Geschtzt-Status der Evaluationsfrage
	 */
	function setProtected( $protected = 1 )
	{
		$this->protected = $protected;
	}

	/**
	 * Liefert die Antworttypen-Id der Evaluationsfrage zurck.
	 * @return int Antworttypen-Id der Evaluationsfrage
	 */
	function getAnswerTypeId()
	{
		return $this->answer_type_id;
	}

	/**
	 * Setzt den Antworttypen-Id der Evaluationsfrage.
	 * @param int $answer_type_id Antworttypen-Id der Evaluationsfrage
	 */
	function setAnswerTypeId( $answer_type_id )
	{
		$this->answer_type_id = $answer_type_id;
	}

	/**
	 * Liefert das nderungsdatum der Evaluationsfrage zurck.
	 * @return int nderungsdatum der Evaluationsfrage als Unix-Timestamp
	 */
	function getChangeDate()
	{
		return $this->change_date;
	}

	/**
	 * Liefert die Position der Evaluationsfrage innerhalb der Gruppe zurck.
	 * @return int Position der Evaluationsfrage
	 */
	function getPosition()
	{
		return $this->position;
	}

	/**
	 * Setzt die Position der Evaluationsfrage innerhalb der Gruppe.
	 * @param int $position Position der Evaluationsfrage
	 */
	function setPosition( $position )
	{
		$this->position = $position;
	}

	/**
	 * Liefert den HTML-Typen der Evaluationsfrage zurck.
	 * @return string HTML-Typ der Evaluationsfrage
	 */
	function getHtmlType()
	{
		return $this->html_type;
	}

	/**
	 * Bricht das Objekt in ein Array auf.
	 * Die Rckgabe sieht dabei folgendermassen aus:<br/>
	 * <pre>
	 *    [
	 *     id             => Id der Evaluationsfrage,
	 *     title          => Titel der Evaluationsfrage,
	 *     text           => Text der Evaluationsfrage,
	 *     protected      => Geschtzt-Status der Evaluationsfrage,
	 *     position       => Position der Evaluationsfrage,
	 *     html_type      => HTML-Typ der Evaluationsfrage,
	 *     answer_type_id => Id des Antworttypen,
	 *     answers        => [
	 *       ...
	 *                       ]{0..x}
	 *    ]
	 * </pre>
	 *
	 * @return array Evaluationsfragendaten
	 * @see ibit_EvaluationAnswer::flattenToArray()
	 */
	function flattenToArray()
	{
		$result = array();
		$result['id'] = $this->evaluation_question_template_id;
		$result['title'] = $this->title;
		$result['text'] = $this->text;
		$result['protected'] = $this->protected;
		$result['position'] = $this->position;
		$result['html_type'] = $this->html_type;
		$result['answer_type_id'] = $this->answer_type_id;

		$result['answers'] = array();
		foreach ( $this->answers as $answer )
		{
			//Nur wenn nicht leer...
			if(!empty($answer))
			{
				array_push( $result['answers'], $answer->flattenToArray() );
			}
		}

		return $result;
	}
}
?>