<?php
/**
 * classes/ibit_Helper.class.php
 *
 * @author  Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @author  Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author  Ivn Marcos <ivan.marcos@gmx.net>
 * @author  Nico Mueller <nico.mueller@uni-oldenburg.de>
 * @package IBIT_Evaluation
 * @copyright Copyright 2006, 2007, 2008 IBIT
 * @version 2.0.10
 */

/**
 * Generelle "Helper"-Klasse
 *
 * Diese Klasse fasst verschiedene statische Funktionen zusammen, die nicht wirklich in den Kontext einer anderen Klasse passen oder an
 * verschiedenen Stellen unterschiedlicher Klassen verwendet werden.
 *
 * @author     Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author     Ivn Marcos <ivan.marcos@gmx.net>
 * @author     Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @package    IBIT_Evaluation
 * @subpackage Helper
 */
class ibit_Helper
{
	/**
	 * Ldt die Fakultts-Id eines Instituts aus der Datenbank.
	 * @param string $institute_id Id des Instituts
	 * @return array Fakultts-Id
	 * @static
	 */
	function getFacultyForInstitute( $institute_id )
	{
		return DBManager::Get()->query("SELECT fakultaets_id FROM Institute WHERE Institut_id = '{$institute_id}'")->fetchColumn();
	}

	/**
	 * Ldt den Namen einer Fakultt aus der Datenbank.
	 * @param string $faculty_id Id der Fakultt
	 * @return string Name der Fakultt
	 * @static
	 */
	function getFacultyName( $faculty_id )
	{
		return DBManager::Get()->query("SELECT Name FROM Institute WHERE Institut_id = fakultaets_id AND fakultaets_id = '{$faculty_id}'")->fetchColumn();
	}

	/**
	 * Ldt den Namen eines Instituts aus der Datenbank.
	 * @param string $institute_id Id des Instituts
	 * @return string Name des Instituts
	 * @static
	 */
	function getInstituteName( $institute_id )
	{
		return DBManager::Get()->query("SELECT Name FROM Institute WHERE Institut_id = '{$institute_id}'")->fetchColumn();
	}

	/**
	 * Liefert die Anzahl an Instituten fr eine bestimmte Fakultt zurck.
	 *
	 * @param int $faculty_id Id der Fakultt
	 * @return int Anzahl der Institute
	 */
	function getInstituteCountByFacultyId( $faculty_id )
	{
		return DBManager::Get()->query("SELECT COUNT(*) FROM Institute WHERE fakultaets_id = '{$faculty_id}'")->fetchColumn();
	}

	/**
	 * Liefert die Anzahl an Seminaren fr eine bestimmte Fakultt und ein bestimmtes Semester zurck.
	 *
	 * @param int $faculty_id Id der Fakultt
	 * @param string $semester_id Id des Semesters
	 * @return int Anzahl der Seminare
	 */
	function getSeminarCountByFacultyId( $faculty_id, $semester_id )
	{
		return DBManager::Get()->query("SELECT COUNT( DISTINCT s.Seminar_id ) FROM seminare AS s LEFT JOIN seminar_inst AS si ON (si.seminar_id=s.seminar_id) LEFT JOIN Institute AS i ON( i.Institut_id IN (s.Institut_id,si.institut_id) ) INNER JOIN semester_data AS sem ON (sem.semester_id='{$semester_id}') WHERE fakultaets_id='{$faculty_id}' AND start_time BETWEEN sem.beginn AND sem.ende")->fetchColumn();
	}

	/**
	 * Liefert die Anzahl an Seminaren fr ein bestimmtes Institut und ein bestimmtes Semester zurck.
	 *
	 * @param int $institute_id Id des Instituts
	 * @param string $semester_id Id des Semesters
	 * @return int Anzahl der Seminare
	 */
	function getSeminarCountByInstituteId( $institute_id, $semester_id )
	{
		return DBManager::Get()->query("SELECT COUNT( DISTINCT s.Seminar_id ) FROM seminare AS s LEFT JOIN seminar_inst AS si ON (si.seminar_id=s.seminar_id) LEFT JOIN Institute AS i ON( i.Institut_id IN (s.Institut_id,si.institut_id) ) INNER JOIN semester_data AS sem ON (sem.semester_id='{$semester_id}') WHERE i.Institut_id='{$institute_id}' AND start_time BETWEEN sem.beginn AND sem.ende")->fetchColumn();
	}

	/**
	 * Ldt den Namen eines Seminars aus der Datenbank.
	 * @param string $seminar_id Die Id des Seminars
	 * @return string Name des Seminars
	 * @static
	 */
	function getSeminarName( $seminar_id )
	{
		$rows = DBManager::Get()->query("SELECT Seminar_id, name FROM seminare WHERE Seminar_id IN ('".implode("','", (array)$seminar_id)."')")->fetchAll(PDO::FETCH_ASSOC);

		$seminars = array();
		foreach ($rows as $row)
		{
			$seminars[ $row['Seminar_id'] ] = $row['name'];
		}

		return is_array($seminar_id)
			? $seminars
			: reset($seminars);
	}

	/**
	 * Liefert eine Liste aller vorhandenen Semester zurck.
	 * Die Rckgabe sieht dabei folgendermassen aus:<br/>
	 * <pre>
	 *    [
	 *     id      => Id des Semesters,
	 *     name    => Name des Semesters,
	 *     current => Gibt an, ob das Semester das aktuelle Semester ist
	 *    ]{0..x}
	 * </pre>
	 *
	 * @return array Semesterdaten
	 * @static
	 */
	function getSemester()
	{
		return DBManager::Get()->query("SELECT semester_id AS id, name, UNIX_TIMESTAMP() BETWEEN beginn AND ende AS currentsemester FROM semester_data ORDER BY beginn DESC")->fetchAll(PDO::FETCH_ASSOC);
	}

	/**
	 * Ldt den Namen eines Semesters aus der Datenbank.
	 * @param string $semester_id Id des Semesters
	 * @return string Name des Semesters
	 * @static
	 */
	function getSemesterName( $semester_id )
	{
		return DBManager::Get()->query("SELECT name FROM semester_data WHERE semester_id = '{$semester_id}'")->fetchColumn();
	}

	/**
	 * Liefert eine Liste aller Semester zurck, in denen (abgelaufene) Evaluationen vorhanden sind.
	 * Die Rckgabe sieht dabei folgendermassen aus:<br/>
	 * <pre>
	 *    [
	 *     id      => Id des Semesters,
	 *     name    => Name des Semesters,
	 *     current => Gibt an, ob das Semester das aktuelle Semester ist
	 *    ]{0..x}
	 * </pre>
	 *
	 * @param boolean $expired Gibt an, ob nur Semester mit abgelaufenen Evaluationen ausgelesen werden sollen
	 * @return array Semesterdaten
	 * @static
	 */
	function getEvaluationSemester( $expired = false )
	{
		$semester_ids = DBManager::Get()->query("SELECT DISTINCT semester_id FROM ibit_evaluation".($expired?" WHERE stop_date < NOW()":''))->fetchAll(PDO::FETCH_COLUMN);
		if ( empty( $semester_ids ) )
		{
			return array();
		}

		return DBManager::Get()->query("SELECT semester_id AS id, name, UNIX_TIMESTAMP() BETWEEN beginn AND ende AS currentsemester FROM semester_data WHERE semester_id IN ('".implode("','", $semester_ids)."') ORDER BY beginn DESC")->fetchAll(PDO::FETCH_ASSOC);
	}

	/**
	 * Liefert eine Liste aller vorhandenen Fakultten zurck.
	 * Die Rckgabe sieht dabei folgendermassen aus:<br/>
	 * <pre>
	 *    [
	 *     fakultaet_id => Id der Fakultt,
	 *     name         => Name der Fakultt
	 *    ]{0..x}
	 * </pre>
	 *
	 * @return array Fakulttsdaten
	 * @static
	 */
	function getFaculties()
	{
		return DBManager::Get()->query("SELECT Institut_id AS fakultaet_id, Name AS name FROM Institute WHERE Institut_id = fakultaets_id ORDER BY Name ASC")->fetchAll(PDO::FETCH_ASSOC);
	}

	/**
	 * Liefert eine Liste aller vorhandenen Institute zurck.
	 * Die Rckgabe sieht dabei folgendermassen aus:<br/>
	 * <pre>
	 *    [
	 *     institute_id => Id des Instituts,
	 *     name         => Name des Instituts
	 *    ]{0..x}
	 * </pre>
	 *
	 * @return array Institutdaten
	 * @static
	 */
	function getInstitutes()
	{
		return DBManager::Get()->query("SELECT Institut_id AS institute_id, Name AS name FROM Institute WHERE Institut_id != fakultaets_id ORDER BY Name ASC")->fetchAll(PDO::FETCH_ASSOC);
	}

	/**
	 * Schtzt eine Evaluationsvorlage vor Vernderungen.
	 * @param int $template_id Die Id der zu schtzenden Evaluationsvorlage
	 * @static
	 * @uses ibit_EvaluationTemplate
	 */
	function protectTemplate( $template_id )
	{
		$template = new ibit_EvaluationTemplate( $template_id );

		foreach ($template->groups as $group)
		{
			$group->setProtected();
			$group->save();

			foreach ($group->questions as $question)
			{
				$question->setProtected();
				$question->save();
			}
		}
		$template->setProtected();
		$template->save();
	}

	/**
	 * Prft einen bergebenen numerischen Wert auf Gltigkeit.
	 *
	 * @param int $number Der zu prfende numerische Wert
	 * @param int $min Das Minimum des Wertes (null fr keine Angabe)
	 * @param int $max Das Maximum des Wertes (null fr keine Angabe)
	 * @param string $description Die Beschreibung des Wertes
	 * @param array $error Container fr Fehlermeldungen (Output-Parameter)
	 * @static
	 */
	function checkNumber( &$number, $min, $max, $description, &$error )
	{
		if ( !isset( $number ) )
		{
			array_push( $error, _('Bitte '.$description.' angeben') );
		}
		elseif ( !is_numeric( $number ) or ( $min !== null and $number < $min ) or ( $max !== null and $number > $max ) )
		{
			unset( $number );
			$error_string = _('Bitte '.$description.' korrekt angeben');
			if ( $min === null )
			{
				$error_string .= ' (<'.$max.')';
			}
			elseif ( $max === null )
			{
				$error_string .= ' (>'.$min.')';
			}
			elseif ( $min !== null and $max !== null )
			{
				$error_string .= ' ('.$min.'..'.$max.')';
			}
			array_push( $error,  $error_string );
		}
	}

	/**
	 * Liefert alle Institute aus der Datenbank als Objekte zurck.
	 * @return array Liste aller Institute als Objekte
	 * @static
	 */
	function getInstituteObjects()
	{
		$rows = DBManager::Get()->query("SELECT LPAD(Institut_id, 32, '0') AS inst_id, Institut_id, Name, fakultaets_id FROM Institute ORDER BY fakultaets_id, inst_id, Name")->fetchAll(PDO::FETCH_ASSOC);

		$institutes = array();
    	foreach ($rows as $row)
    	{
    		$instid = $row['Institut_id'];
    		$parentid = $row['fakultaets_id'];
    		$name = $row['Name'];

    		if ( $instid == $parentid )
			{
    			$institute = new StudIPInstitute();
    			$institute->setId($instid);
    			$institute->setName($name);
    			if ( isset( $institutes[$instid] ) and is_object( $institutes[$instid] ) )
				{
    				$children = $institutes[ $instid ]->getAllChildInstitutes();
    				foreach ( $children as $child )
					{
    					$institute->addChild( $child );
    				}
    			}
    			$institutes[ $instid ] = $institute;
    		}
    		else
			{
    			$child = new StudIPInstitute();
    			$child->setId($instid);
    			$child->setName($name);
    			if ( !isset( $institutes[$parentid] ) or !is_object( $institutes[$parentid] ) )
				{
    				$institute = new StudIPInstitute();
    			}
				else
				{
					$institute = $institutes[ $parentid ];
				}
    			$institute->addChild( $child );
    			$institutes[ $parentid ] = $institute;
    		}
    	}

    	return $institutes;
	}

	/**
	 * Liefert ein Institut aus der Datenbank als Objekt zurck.
	 * @param string $institute_id Id des Instituts
	 * @return object Das Institut als Objekt
	 * @static
	 * @uses ibit_Helper::getInstituteByFacultyId()
	 */
	function getInstituteObject( $institute_id )
	{
		$row = DBManager::Get()->query("SELECT Institut_id, Name, fakultaets_id FROM Institute WHERE Institut_id='{$institute_id}'")->fetch(PDO::FETCH_ASSOC);
		if (!$row)
		{
			return false;
		}

		$institute = new StudIPInstitute();
		$institute->setId( $row['Institut_id'] );
		$institute->setName( $row['Name'] );
		if ( $row['fakultaets_id'] == $row['Institut_id'] )
		{
			$institutes = self::getInstituteByFacultyId( $row['fakultaets_id'] );
			foreach ( $institutes as $child )
			{
				$institute->addChild( $child );
			}
		}

    	return $institute;
	}

	/**
	 * Liefert die zu einer Fakultten gehrenden Institute als Objekte zurck.
	 * @param string $faculty_id Id der Fakultt
	 * @return array Liste der Institute als Objekte
	 * @static
	 */
	function getInstituteByFacultyId( $faculty_id )
	{
		$rows = DBManager::Get()->query("SELECT Institut_id, Name, fakultaets_id FROM Institute WHERE fakultaets_id='{$faculty_id}' AND Institut_id <> fakultaets_id")->fetchAll(PDO::FETCH_ASSOC);

    	$institutes = array();
    	foreach ($rows as $row)
    	{
    		$institute = new StudIPInstitute();
    		$institute->setId( $row['Institut_id'] );
    		$institute->setName( $row['Name'] );

    		array_push( $institutes, $institute );
    	}

    	return $institutes;
	}

	/**
	 * Liefert die Seminare eines Instituts in einem bestimmten Semester zurck.
	 * Die Rckgabe sieht dabei wie folgt aus:
	 *
	 * <pre>
	 *    [
	 *      id    => Id des Seminars,
	 *      titel => Titel des Seminars
	 * 		typ   => Art der Veranstaltung
	 *    ]{0..x}
	 * </pre>
	 *
	 * @param string $institute_id Id des Instituts
	 * @param string $semester_id Id des Semesters
	 * @return array Liste der Seminare
	 * @static
	 */
	function getSeminarsForInstitute( $institute_id, $semester_id )
	{
		$courses = array();

		// aufgrund fehlerhafter institut_ids muss selbst die id berprft werden...
		if(strlen($institute_id) == 32 && substr($institute_id, 0, 1) == 0)
		{
			$trimmed = ltrim($institute_id, "0");

			$courses = DBManager::Get()->query("SELECT se.Seminar_id AS id, se.Name AS titel, se.status AS typ, se.visible FROM seminar_inst s JOIN seminare se ON (s.seminar_id=se.Seminar_id) WHERE s.institut_id='{$trimmed}' AND se.start_time BETWEEN (SELECT beginn FROM semester_data WHERE semester_id='{$semester_id}') AND (SELECT ende FROM semester_data WHERE semester_id='{$semester_id}') UNION SELECT sem.Seminar_id, sem.Name, sem.status, sem.visible FROM seminare sem WHERE sem.institut_id='{$trimmed}' AND sem.start_time BETWEEN (SELECT beginn FROM semester_data WHERE semester_id='{$semester_id}') AND (SELECT ende FROM semester_data WHERE semester_id='{$semester_id}')")->fetchAll(PDO::FETCH_ASSOC);
		}

		$courses = array_merge($courses, DBManager::Get()->query("SELECT se.Seminar_id AS id, se.Name AS titel, se.status AS typ, se.visible FROM seminar_inst s JOIN seminare se ON (s.seminar_id=se.Seminar_id) WHERE s.institut_id='{$institute_id}' AND se.start_time BETWEEN (SELECT beginn FROM semester_data WHERE semester_id='{$semester_id}') AND (SELECT ende FROM semester_data WHERE semester_id='{$semester_id}') UNION SELECT sem.Seminar_id, sem.Name, sem.status, sem.visible FROM seminare sem WHERE sem.institut_id='{$institute_id}' AND sem.start_time BETWEEN (SELECT beginn FROM semester_data WHERE semester_id='{$semester_id}') AND (SELECT ende FROM semester_data WHERE semester_id='{$semester_id}')")->fetchAll(PDO::FETCH_ASSOC));

		return $courses;
	}

	/**
	 * Eigene Buttons einbinden
	 *
	 * @param string $name
	 * @param string $mode
	 * @param string $tooltip
	 * @param boolean $inputname
	 * @return string HTML-Code des Buttons
	 */
	function makeButton($name, $mode = "img", $tooltip = false, $inputname = false, $folder = false)
	{
		$name = str_replace( Array('','','','',' '), Array('ae','oe','ue','ss',''), strtolower( $name ) );
		if ($folder == false)
		{
			$url = $GLOBALS['ibit_eval_pluginpath'].'/images/'.$name.'-button.png';
		}
		elseif($folder == 'assets_image')
		{
			$url = $GLOBALS['ASSETS_URL'].'images/'.$name.'.gif';
		}
		else
		{
			$url = localeButtonUrl($name . '-button.png');
		}

		$tooltext = ($tooltip ? tooltip($tooltip) : '');

		switch ($mode) {

			case 'img':
				$tag = "\n" . sprintf('<img src="%s" %s border="0" align="absmiddle" />',
				                      $url, $tooltext);
				break;


			case 'input':
				$tag = "\n" . sprintf('<input type="image" src="%s" %s border="0" '.
				                      'name="%s" align="absmiddle" />',
			                        $url, $tooltext, $inputname);
				break;


			default:
				$tag = sprintf('class="button" src="%s"', $url);

		}
		return $tag;
	}

	function confirm($name, $description, $message, $yes_link, $no_link, $base_link, $folder)
	{
		$image = ibit_Helper::makeButton($name, 'img', $description, false, $folder);

		$result = sprintf('<a href="%s&confirm=this&message=%s&yes=%s&no=%s" onclick="if (confirm(\'%s\')) location.href=\'%s\'; return false;">%s</a>',
			$base_link,
			urlencode($message),
			urlencode($base_link.'&'.$yes_link),
			urlencode($base_link.'&'.$no_link),
			strip_tags($message),
			$base_link.'&'.$yes_link,
			$image
		);

		return $result;

	}
}
?>